package hpjava.compiler.abstractexp;
 
import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;
 
import hpjava.compiler.reflection.*;

import java.util.*;
import java.io.*;

public abstract class AbstractExpression extends AbstractNode {

  abstract public void accept(AbstractExpressionVisitor v);

  public AssignmentNode assignment(AbstractExpression r){
    return new AssignmentNode(this, r);
  }

  public MultiplicationAssignmentNode multiplicationAssignment(AbstractExpression r){
    return new MultiplicationAssignmentNode(this, r);
  }

  public DivisionAssignmentNode divisionAssignment(AbstractExpression r){
    return new DivisionAssignmentNode(this, r);
  }

  public ModulusAssignmentNode modulusAssignment(AbstractExpression r){
    return new ModulusAssignmentNode(this, r);
  }

  public AdditionAssignmentNode additionAssignment(AbstractExpression r){
    return new AdditionAssignmentNode(this, r);
  }

  public SubtractionAssignmentNode subtractionAssignment(AbstractExpression r){
    return new SubtractionAssignmentNode(this, r);
  }

  public LeftShiftAssignmentNode leftShiftAssignment(AbstractExpression r){
    return new LeftShiftAssignmentNode(this, r);
  }

  public RightShiftAssignmentNode rightShiftAssignment(AbstractExpression r){
    return new RightShiftAssignmentNode(this, r);
  }

  public ExtensionRightShiftAssignmentNode extensionRightShiftAssignment(AbstractExpression r){
    return new ExtensionRightShiftAssignmentNode(this, r);
  }

  public AndAssignmentNode andAssignment(AbstractExpression r){
    return new AndAssignmentNode(this, r);
  }

  public ExclusiveOrAssignmentNode exclusiveOrAssignment(AbstractExpression r){
    return new ExclusiveOrAssignmentNode(this, r);
  }

  public InclusiveOrAssignmentNode inclusiveOrAssignment(AbstractExpression r){
    return new InclusiveOrAssignmentNode(this, r);
  }

  public ConditionalNode conditional(AbstractExpression r1, AbstractExpression r2){
    return new ConditionalNode(this, r1, r2);
  }

  public ConditionalOrNode conditionalOr(AbstractExpression r){
    return new ConditionalOrNode(this, r);
  }

  public ConditionalAndNode conditionalAnd(AbstractExpression r){
    return new ConditionalAndNode(this, r);
  }

  public InclusiveOrNode inclusiveOr(AbstractExpression r){
    return new InclusiveOrNode(this, r);
  }

  public ExclusiveOrNode exclusiveOr(AbstractExpression r){
    return new ExclusiveOrNode(this, r);
  }

  public AndNode and(AbstractExpression r){
    return new AndNode(this, r);
  }

  public EqualityNode equality(AbstractExpression r){
    return new EqualityNode(this, r);
  }

  public InEqualityNode inEquality(AbstractExpression r){
    return new InEqualityNode(this, r);
  }

  public InstanceOfNode instanceOf(TypeNode t){
    return new InstanceOfNode(this, t);
  }

  public GreaterNode greater(AbstractExpression r){
    return new GreaterNode(this, r);
  }

  public LessNode less(AbstractExpression r){
    return new LessNode(this, r);
  }

  public GreaterThanEqNode greaterThanEq(AbstractExpression r){
    return new GreaterThanEqNode(this, r);
  }

  public LessThanEqNode lessThanEq(AbstractExpression r){
    return new LessThanEqNode(this, r);
  }

  public LeftShiftNode leftShift(AbstractExpression r){
    return new LeftShiftNode(this, r);
  }

  public RightShiftNode rightShift(AbstractExpression r){
    return new RightShiftNode(this, r);
  }

  public ExtensionRightShiftNode extensionRightShift(AbstractExpression r){
    return new ExtensionRightShiftNode(this, r);
  }

  public AdditiveNode add(AbstractExpression r){
    return new AdditiveNode(this, r);
  }

  public SubtractiveNode subtract(AbstractExpression r){
    return new SubtractiveNode(this, r);
  }

  public MultiplicativeNode multiple(AbstractExpression r){
    return new MultiplicativeNode(this, r);
  }

  public DivideNode divide(AbstractExpression r){
    return new DivideNode(this, r);
  }

  public ModuleNode module(AbstractExpression r){
    return new ModuleNode(this, r);
  }

  public PlusNode plus(){
    return new PlusNode(this);
  }

  public MinusNode minus(){
    return new MinusNode(this);
  }

  public PreIncrementNode preIncrement(){
    return new PreIncrementNode(this);
  }

  public PreDecrementNode preDecrement(){
    return new PreDecrementNode(this);
  }

  public ComplementNode complement(){
    return new ComplementNode(this);
  }

  public LogicalNotNode logicalNot(){
    return new LogicalNotNode(this);
  }

  public PostfixNode postfix(){
    return new PostfixNode(this);
  }

  public CastNode cast(TypeNode t){
    return new CastNode(this, t);
  }

  public GroupNode group(AbstractExpression r){
    return new GroupNode(this, r);
  }

  public PostfixAdditionNode postfixAddition(){
    return new PostfixAdditionNode(this);
  }

  public PostfixSubtractionNode postfixSubtraction(){
    return new PostfixSubtractionNode(this);
  }

  public Expression toExpression() {
    Expression empty = new Expression(null, new NodeOptional());

    ExpressionManager.register(empty, this);

    return empty;
  }

  public StatementExpression toStatementExpression() {
    StatementExpression empty = new StatementExpression(new NodeChoice(null, -1));

    ExpressionManager.register(empty, this);

    return empty;
  }

  public void typeChecking() {
    // the function is also called from manager directly ...

    if (!this.typeChecked) {
      AbstractExpressionInspector inspector = 
        new AbstractExpressionInspector();

      this.accept(inspector);
    }

  }

}


