/*
 * File         : AbstractPrimaryExpression.java
 * Author       : Guansong Zhang
 * Created      : Wed Aug 12 14:21:46 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1999/03/16 20:23:26 $
 */

package hpjava.compiler.abstractexp;

import hpjava.compiler.syntaxtree.*;

import hpjava.compiler.reflection.*;


abstract public class AbstractPrimaryExpression extends AbstractNode {
  abstract public void accept (AbstractPrimaryExpressionVisitor v);

  public PrimaryConcatenation append (PrimaryItem leaf) {
    PrimaryConcatenation concatenation = 
      new PrimaryConcatenation(this, leaf);
    
    this.setParent(concatenation);
    leaf.setParent(concatenation);

    return concatenation;

  }

  public void typeChecking() {
    // the function is also called from manager directly ...

    if (!this.typeChecked) {
      AbstractPrimaryExpressionInspector inspector = 
        new AbstractPrimaryExpressionInspector();

      this.accept(inspector);
    }

  }

  private PrimaryConcatenation parent = null;

  protected void setParent(PrimaryConcatenation parent) {
    this.parent = parent;
  }

  public PrimaryConcatenation getParent() {
    return parent;
  }

  private Node registration = null;

  protected void setRegistration(Node n) {
    this.registration = n;
  }

  protected Node getRegistration() {
    return registration;
  }

  public void replaceWith(AbstractPrimaryExpression exp) {

    if (this.parent == null) {
      if (this.registration == null)
        throw new Error();

      PrimaryExpressionManager.register(this.registration, exp);

    }
    else {
      if (parent.prefix == this) {
        this.parent.prefix = exp;
      }
      else if (parent.suffix == this) {
        this.parent.suffix = (PrimaryItem)exp ;
      }
      else {
        throw new Error();
      }
    }
  }

  public PrimaryExpression toPrimaryExpression() {
    PrimaryExpression empty = new PrimaryExpression(null, new NodeListOptional());
    
    PrimaryExpressionManager.register(empty, this);

    return empty;
  }

  public AbstractExpression toAbstractExpression() {
    PrimaryExpression primary = this.toPrimaryExpression();

    PrimaryExpressionNode aexp = new PrimaryExpressionNode(primary);

    return aexp;
  }

  public Expression toExpression() {
    return this.toAbstractExpression().toExpression();    
  }

}
