/*
 * File         : DistArraySecRefDimListDescriptor.java
 * Author       : Guansong Zhang
 * Created      : Wed Aug 12 14:23:07 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1999/03/16 20:23:43 $
 */

package hpjava.compiler.abstractexp;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

import hpjava.compiler.reflection.*;

public class DistArraySecRefDimListDescriptor extends PrimaryItem {
  public DistArraySecRefDimList leaf;

  public DistArraySecRefDimListDescriptor (DistArraySecRefDimList leaf) {
    this.leaf = leaf;
  }

  public void accept(AbstractPrimaryExpressionVisitor v) {
    v.visit(this);
  }

  public int numberOfDims() { 

    DistArraySecRefDimList list = this.leaf; 
     
    if (list.f1.present()) { 
      return 1+( (NodeListOptional)list.f1 ).size(); 
    } 
    else { 
      return 1; 
    }     
  } 

  public DimensionDescriptor dim(int i) { 
    if (i==0) { 
      DistArraySecRefDim dasr = this.leaf.f0; 
 
      if (dasr.f0.which==0) { 
        return new TripletDescriptor((Triplet)dasr.f0.choice); 
      } 
      else if (dasr.f0.which==1) { 
        return new ExpressionDescriptor((Expression)dasr.f0.choice); 
      } 
      else if (dasr.f0.which==2) { 
        return new KernelSubrangeDescriptor((KernelSubrange)dasr.f0.choice); 
      } 
      else { 
        throw new Error(); 
      } 
    } 
    else { 
      NodeSequence sequence = (NodeSequence)this.leaf.f1.elementAt(i-1); 
      DistArraySecRefDim dasr = (DistArraySecRefDim)sequence.elementAt(1); 
 
      if (dasr.f0.which==0) { 
        return new TripletDescriptor((Triplet)dasr.f0.choice); 
      } 
      else if (dasr.f0.which==1) { 
        return new ExpressionDescriptor((Expression)dasr.f0.choice); 
      } 
      else if (dasr.f0.which==2) { 
        return new KernelSubrangeDescriptor((KernelSubrange)dasr.f0.choice); 
      } 
      else { 
        throw new Error(); 
      } 
    } 
  } 

  public boolean isTripletDim(int i) {
    if (i==0) {
      DistArraySecRefDim dasrd = this.leaf.f0;

      if (dasrd.f0.which==0) {
        return true;
      }
      else if (dasrd.f0.which==1) {
        return false;
      }
      else if (dasrd.f0.which==2) {
        return false;
      }
      else {
        throw new Error();
      }
    }
    else {
      NodeSequence sequence = (NodeSequence)this.leaf.f1.elementAt(i-1);
      DistArraySecRefDim dasrd = (DistArraySecRefDim)sequence.elementAt(1);

      if (dasrd.f0.which==0) {
        return true;
      }
      else if (dasrd.f0.which==1) {
        return false;
      }
      else if (dasrd.f0.which==2) {
        return false;
      }
      else {
        throw new Error();
      }
    }
  }

  public boolean isExpressionDim(int i) {
    if (i==0) {
      DistArraySecRefDim dasrd = this.leaf.f0;

      if (dasrd.f0.which==0) {
        return false;
      }
      else if (dasrd.f0.which==1) {
        return true;
      }
      else if (dasrd.f0.which==2) {
        return false;
      }
      else {
        throw new Error();
      }
    }
    else {
      NodeSequence sequence = (NodeSequence)this.leaf.f1.elementAt(i-1);
      DistArraySecRefDim dasrd = (DistArraySecRefDim)sequence.elementAt(1);

      if (dasrd.f0.which==0) {
        return false;
      }
      else if (dasrd.f0.which==1) {
        return true;
      }
      else if (dasrd.f0.which==2) {
        return false;
      }
      else {
        throw new Error();
      }
    }
  }

  public boolean isKernelSubrangeDim(int i) {
    if (i==0) {
      DistArraySecRefDim dasrd = this.leaf.f0;

      if (dasrd.f0.which==0) {
        return false;
      }
      else if (dasrd.f0.which==1) {
        return false;
      }
      else if (dasrd.f0.which==2) {
        return true;
      }
      else {
        throw new Error();
      }
    }
    else {
      NodeSequence sequence = (NodeSequence)this.leaf.f1.elementAt(i-1);
      DistArraySecRefDim dasrd = (DistArraySecRefDim)sequence.elementAt(1);

      if (dasrd.f0.which==0) {
        return false;
      }
      else if (dasrd.f0.which==1) {
        return false;
      }
      else if (dasrd.f0.which==2) {
        return true;
      }
      else {
        throw new Error();
      }
    }
  }

  // set flag, and return the number of 1s.

  private int flagCounter(int [] flag,
                          DistArraySecRefDimListDescriptor d) {

    int counter = 0;

    for (int i = 0; i<d.numberOfDims(); i++) {
      if (d.isTripletDim(i)) {
        flag[i]=1;
        counter++;
      }
      else if (d.isExpressionDim(i)) {
        AbstractExpression aexp =
          (AbstractExpression) ExpressionManager.
          lookup(( (ExpressionDescriptor)d.dim(i) ).leaf);

        ClassReflector reflector = aexp.getClassReflector();

        if (reflector.getName().equals("hpjava.lang.Range")) {
          flag[i]=1;
          counter++;
        }
        else {
          flag[i]=0;
        }
      }
      else if (d.isKernelSubrangeDim(i)) {
        throw new Error("not finished");
      }
    }

    return counter;

  }

  public String secRefSignatureOf(ClassReflector cr) {

    if (!cr.isDistArray())
      throw new Error();

    // the following part need to be changed.
    ClassReflectorDescriptor crd
      = ((ClassReflectorByDescriptor)cr).descriptor();

    ArrayReflectorDescriptor ard = (ArrayReflectorDescriptor) crd;

    int [] flag = new int [numberOfDims()];

    int dim = flagCounter(flag, this);

    StringBuffer nameBuffer = new StringBuffer("]"+dim);

    for (int i = 0; i<numberOfDims(); i++) {
      if (flag[i]==1) {
        if (ard.isCollapsedDim(i))
          nameBuffer.append('c');
        else
          nameBuffer.append('d');
      }
    }

    return nameBuffer.toString();
  }

}
