/*
 * File         : ExpressionManager.java
 * Author       : Guansong Zhang
 * Created      : Wed Aug 12 13:03:10 1998
<<<<<<< ExpressionManager.java
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1999/03/17 20:33:24 $
=======
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1999/03/17 20:33:24 $
>>>>>>> 1.15
 */

package hpjava.compiler.abstractexp;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

import hpjava.compiler.parser.*;
import hpjava.compiler.unparser.*;
import hpjava.compiler.util.*;

import java.util.*;
import java.lang.*;
import java.io.*;

public class ExpressionManager{
   public static Hashtable table = new Hashtable();

   public ExpressionManager(Node r){
     ExpressionFinder finder = new ExpressionFinder(table);
     r.accept(finder);
     
     //System.out.println("ec stack "+ ec.nodeStack.size());

   }

  public ExpressionManager (Project prj) {
    ExpressionFinder finder = new ExpressionFinder(table);

    Enumeration enum = prj.rootFileTable.keys();

  LOOP: while (enum.hasMoreElements()) {
      Node root = (Node) enum.nextElement();
      root.accept(finder);
    }

    if (prj.isVerbose()) {
      //System.err.println("Node scope: "+nodeScopeTable+" created.");
      System.err.println("Expression table created.");
      System.err.println();
    }
  }

  public static AbstractExpression lookup(Node n){
    return (AbstractExpression)table.get(n);
  }

  public static void register(Node n, AbstractExpression exp) {
    table.put(n, exp);
  }
}

class ExpressionFinder extends ExpressionFindingVisitor{
  protected Hashtable refHashtable;

  final int DEFAULT = 0;
  final int RECURSIVE = 1;

  private IntStack status = new IntStack();
  
  public ExpressionFinder(Hashtable table) {
    this.refHashtable = table;
    status.push(DEFAULT);
  }

   //
   // f0 -> PrimaryExpression()
   // f1 -> ( "/" PrimaryExpression() )*
   //
  public void visit(GroupExpression n){
    ExpressionConvertor convertor =
      new ExpressionConvertor(this.refHashtable);

    n.accept(convertor);

    n.f0.accept(this);

    if (n.f1.present())
      n.f1.accept(this);
  }

   //
   // f0 -> ConditionalExpression()
   // f1 -> [ AssignmentOperator() Expression() ]
   //
  public void visit(Expression n) {
    if (status.peek() == DEFAULT) {

      ExpressionConvertor convertor =
        new ExpressionConvertor(this.refHashtable);

      n.accept(convertor);

      // go on to find 

      n.f0.accept(this); 
 
      if (n.f1.present()) {
        status.push(RECURSIVE);
        n.f1.accept(this); 
        status.pop();
      }

    } 
    else if (status.peek()==RECURSIVE) {
      status.push(DEFAULT);
      n.f0.accept(this);
      status.pop();

      if (n.f1.present()) {
        n.f1.accept(this);
      }
    }
    else {
      throw new Error("Invalid status.");
    }

  }

   //
   // f0 -> PreIncrementExpression()
   //       | PreDecrementExpression()
   //       | PrimaryExpression() [ "++" | "--" | AssignmentOperator() Expression() ]
   //

  public void visit(StatementExpression n) {
    ExpressionConvertor convertor =
      new ExpressionConvertor(this.refHashtable);

    n.accept(convertor);

    n.f0.accept(this);
  }


}


class ExpressionConvertor extends DepthFirstVisitor{
   protected PrintWriter out = new PrintWriter(System.out, true);

   protected Stack nodeStack = new Stack();

   protected Hashtable refHashtable;

   protected int op = 0;

   protected Node startPoint;
   private IntStack status=new IntStack();

   final int DEFAULT = 0;
   final int RECURSIVE = 1;

   public ExpressionConvertor(Hashtable ht){
     //out = new PrintWriter(System.out, true);
     refHashtable = ht;

     status.push(DEFAULT);
   }

   public void flushWriter(){
     out.flush();
   }

  /*
   //
   // A few convenience methods overloading visit()
   //
   public void visit(NodeList n, String sep) {
      for ( Enumeration e = n.elements(); e.hasMoreElements(); ) {
         ((Node)e.nextElement()).accept(this);
         if ( e.hasMoreElements() ) out.print(sep);
      }
   }

   public void visit(NodeListOptional n, String sep) {
      if ( n.present() )
         for ( Enumeration e = n.elements(); e.hasMoreElements(); ) {
            ((Node)e.nextElement()).accept(this);
            if ( e.hasMoreElements() ) out.print(sep);
         }
   }

   public void visit(NodeOptional n, String sep) {
      if ( n.present() ) {
         n.node.accept(this);
         out.print(sep);
      }
   }

   //
   // For convenience, trusts that the node passed to it is a NodeSequence.
   // (of course, will throw an exception if it isn't).
   //
   public void visit(Node n1, String sep) {
      NodeSequence n = (NodeSequence)n1;
      for ( Enumeration e = n.elements(); e.hasMoreElements(); ) {
         ((Node)e.nextElement()).accept(this);
         if ( e.hasMoreElements() ) out.print(sep);
      }
   }

   public void visit(NodeToken n) {
      out.print(n.tokenImage + " ");
   }
  */


  // start to override visit method 

   //
   // f0 -> ConditionalExpression()
   // f1 -> [ AssignmentOperator() Expression() ]
   //
   public void visit(Expression n) {
      n.f0.accept(this);
      AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

      if(n.f1.present()){

        status.push(RECURSIVE);
        n.f1.accept(this);
        status.pop();

        switch( op ){

          case 0:
             tmp = tmp.assignment((AbstractExpression)nodeStack.pop());
             break;

          case 1:
             tmp = tmp.multiplicationAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 2:
             tmp = tmp.divisionAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 3:
             tmp = tmp.modulusAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 4:
             tmp = tmp.additionAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 5:
             tmp = tmp.subtractionAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 6:
             tmp = tmp.leftShiftAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 7:
             tmp = tmp.rightShiftAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 8:
             tmp = tmp.extensionRightShiftAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 9:
             tmp = tmp.andAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 10:
             tmp = tmp.exclusiveOrAssignment((AbstractExpression)nodeStack.pop());
             break;

          case 11:
             tmp = tmp.inclusiveOrAssignment((AbstractExpression)nodeStack.pop());
             break;

          default:
             break;
        }

     }

      if (status.peek()==DEFAULT) {
        refHashtable.put(n, tmp);
      }
      else if (status.peek()==RECURSIVE) {
        nodeStack.push(tmp);
      }
      else {
        throw new Error("Invalid status.");
      }

   }

   //
   // f0 -> "="
   //       | "*="
   //       | "/="
   //       | "%="
   //       | "+="
   //       | "-="
   //       | "<<="
   //       | ">>="
   //       | ">>>="
   //       | "&="
   //       | "^="
   //       | "|="
   //
   public void visit(AssignmentOperator n) {
      n.f0.accept(this);
      op = n.f0.which;
   }

   //
   // f0 -> ConditionalOrExpression()
   // f1 -> [ "?" Expression() ":" ConditionalExpression() ]
   //
   public void visit(ConditionalExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

        NodeSequence seq = (NodeSequence)n.f1.node;

        status.push(RECURSIVE);
        seq.elementAt(1).accept(this);
        status.pop();

        AbstractExpression tmp2 = (AbstractExpression)nodeStack.pop();

        seq.elementAt(3).accept(this);
        AbstractExpression tmp3 = (AbstractExpression)nodeStack.pop();

        tmp = tmp.conditional(tmp2, tmp3);
        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> ConditionalAndExpression()
   // f1 -> ( "||" ConditionalAndExpression() )*
   //
   public void visit(ConditionalOrExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){
          tmp = tmp.conditionalOr((AbstractExpression)nodeStack.pop());
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> InclusiveOrExpression()
   // f1 -> ( "&&" InclusiveOrExpression() )*
   //
   public void visit(ConditionalAndExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){
          tmp = tmp.conditionalAnd((AbstractExpression)nodeStack.pop());
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> ExclusiveOrExpression()
   // f1 -> ( "|" ExclusiveOrExpression() )*
   //
   public void visit(InclusiveOrExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){
          tmp = tmp.inclusiveOr((AbstractExpression)nodeStack.pop());
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> AndExpression()
   // f1 -> ( "^" AndExpression() )*
   //
   public void visit(ExclusiveOrExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){
          tmp = tmp.exclusiveOr((AbstractExpression)nodeStack.pop());
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> EqualityExpression()
   // f1 -> ( "&" EqualityExpression() )*
   //
   public void visit(AndExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){
          tmp = tmp.and((AbstractExpression)nodeStack.pop());
        }

        nodeStack.push(tmp);
      }
   }

   //
   // f0 -> InstanceOfExpression()
   // f1 -> ( ( "==" | "!=" ) InstanceOfExpression() )*
   //
   public void visit(EqualityExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){

          NodeSequence seq = (NodeSequence)n.f1.elementAt(i);

          if(((NodeChoice)seq.elementAt(0)).which == 0){
            tmp = tmp.equality((AbstractExpression)nodeStack.pop());
          }
          else{
            tmp = tmp.inEquality((AbstractExpression)nodeStack.pop());
          }
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> RelationalExpression()
   // f1 -> [ "instanceof" Type() ]
   //
   public void visit(InstanceOfExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        NodeSequence seq = (NodeSequence)n.f1.node;
        seq.elementAt(1).accept(this);

//        TypeNode type = (TypeNode)seq.elementAt(1);
        TypeNode type = new TypeNode((Type)seq.elementAt(1));

        tmp = tmp.instanceOf(type);

        nodeStack.push(tmp);
      }
   }

   //
   // f0 -> ShiftExpression()
   // f1 -> ( ( "<" | ">" | "<=" | ">=" ) ShiftExpression() )*
   //
   public void visit(RelationalExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){

          NodeSequence seq = (NodeSequence)n.f1.elementAt(i);

          if(((NodeChoice)seq.elementAt(0)).which == 0){
            tmp = tmp.less((AbstractExpression)nodeStack.pop());
          }
          else if(((NodeChoice)seq.elementAt(0)).which == 1){
            tmp = tmp.greater((AbstractExpression)nodeStack.pop());
          }
          else if(((NodeChoice)seq.elementAt(0)).which == 2){
            tmp = tmp.lessThanEq((AbstractExpression)nodeStack.pop());
          }
          else{
            tmp = tmp.greaterThanEq((AbstractExpression)nodeStack.pop());
          }
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> AdditiveExpression()
   // f1 -> ( ( "<<" | ">>" | ">>>" ) AdditiveExpression() )*
   //
   public void visit(ShiftExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){

          NodeSequence seq = (NodeSequence)n.f1.elementAt(i);

          if(((NodeChoice)seq.elementAt(0)).which == 0){
            tmp = tmp.leftShift((AbstractExpression)nodeStack.pop());
          }
          else if(((NodeChoice)seq.elementAt(0)).which == 1){
            tmp = tmp.rightShift((AbstractExpression)nodeStack.pop());
          }
          else{
            tmp = tmp.extensionRightShift((AbstractExpression)nodeStack.pop());
          }
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> MultiplicativeExpression()
   // f1 -> ( ( "+" | "-" ) MultiplicativeExpression() )*
   //
   public void visit(AdditiveExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){

          NodeSequence seq = (NodeSequence)n.f1.elementAt(i);

          if(((NodeChoice)seq.elementAt(0)).which == 0){
            tmp = tmp.add((AbstractExpression)nodeStack.pop());
          }
          else{
            tmp = tmp.subtract((AbstractExpression)nodeStack.pop());
          }
        }

        nodeStack.push(tmp);
      }
   }

   //
   // f0 -> UnaryExpression()
   // f1 -> ( ( "*" | "/" | "%" ) UnaryExpression() )*
   //
   public void visit(MultiplicativeExpression n) {
      n.f0.accept(this);

      if(n.f1.present()){
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){

          NodeSequence seq = (NodeSequence)n.f1.elementAt(i);

          if(((NodeChoice)seq.elementAt(0)).which == 0){
            tmp = tmp.multiple((AbstractExpression)nodeStack.pop());
          }
          else if(((NodeChoice)seq.elementAt(0)).which == 1){
            tmp = tmp.divide((AbstractExpression)nodeStack.pop());
          }
          else{
            tmp = tmp.module((AbstractExpression)nodeStack.pop());
          }
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> ( "+" | "-" ) UnaryExpression()
   //       | PreIncrementExpression()
   //       | PreDecrementExpression()
   //       | UnaryExpressionNotPlusMinus()
   //
   public void visit(UnaryExpression n) {
      if(n.f0.which != 0){
        n.f0.accept(this);
      }
      else{
        NodeSequence seq = (NodeSequence)n.f0.choice;
        seq.elementAt(1).accept(this);
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

        if(((NodeChoice)seq.elementAt(0)).which == 0){
          tmp = tmp.plus();
        }
        else{
          tmp = tmp.minus();
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> "++"
   // f1 -> PrimaryExpression()
   //
   public void visit(PreIncrementExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

      tmp = tmp.preIncrement();
      nodeStack.push(tmp);
   }

   //
   // f0 -> "--"
   // f1 -> PrimaryExpression()
   //
   public void visit(PreDecrementExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

      tmp = tmp.preDecrement();
      nodeStack.push(tmp);
   }

   //
   // f0 -> ( "~" | "!" ) UnaryExpression()
   //       | CastExpression()
   //       | PostfixExpression()
   //
   public void visit(UnaryExpressionNotPlusMinus n) {
      if(n.f0.which != 0){
        n.f0.accept(this);
      }
      else{
        NodeSequence seq = (NodeSequence)n.f0.choice;
        seq.elementAt(1).accept(this);
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

        if(((NodeChoice)seq.elementAt(0)).which == 0){
          tmp = tmp.complement();
        }
        else{
          tmp = tmp.logicalNot();
        }

        nodeStack.push(tmp);
      }

   }

   //
   // f0 -> PrimaryExpression()
   // f1 -> [ "++" | "--" ]
   //
   public void visit(PostfixExpression n) {
      n.f0.accept(this);
      AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

      if(n.f1.present()){
        n.f1.accept(this);
        System.out.println("Hi...");
        tmp = tmp.postfix();
      }

      nodeStack.push(tmp);
   }

   //
   // f0 -> "(" Type() ")" UnaryExpression()
   //       | "(" Type() ")" UnaryExpressionNotPlusMinus()
   //
   public void visit(CastExpression n) {
      NodeSequence seq = (NodeSequence)n.f0.choice;
      seq.elementAt(3).accept(this);
      AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

//      TypeNode type = (TypeNode)seq.elementAt(1);

      TypeNode type = new TypeNode((Type)seq.elementAt(1));

      tmp = tmp.cast(type);
      nodeStack.push(tmp);
   }

   //
   // f0 -> PrimaryPrefix()
   // f1 -> ( PrimarySuffix() )*
   //
   public void visit(PrimaryExpression n) {
      PrimaryExpressionNode pl = new PrimaryExpressionNode(n);
      AbstractExpression rtn = pl;

      nodeStack.push(rtn);

      //      n.f0.accept(this);
      //      n.f1.accept(this);
   }


   //
   // f0 -> PrimaryExpression()
   // f1 -> ( "/" PrimaryExpression() )*
   //
   public void visit(GroupExpression n) {
      n.f0.accept(this);
      AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

      if(n.f1.present()){

        for(int i=n.f1.size()-1; i>=0;i--){
          n.f1.elementAt(i).accept(this);
        }

        for(int i=0; i<n.f1.size(); i++){
          tmp = tmp.group((AbstractExpression)nodeStack.pop());
        }

        nodeStack.push(tmp);
      }

      //     nodeStack.push(tmp);

     refHashtable.put(n, tmp);

   }


   //
   // f0 -> PreIncrementExpression()
   //       | PreDecrementExpression()
   //       | PrimaryExpression() [ "++" | "--" | AssignmentOperator() Expression() ]
   //
   public void visit(StatementExpression n) {

      if(n.f0.which != 2){
        n.f0.accept(this);
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();
        nodeStack.push(tmp);

        refHashtable.put(n, tmp);
      }
      else{
        NodeSequence seq = (NodeSequence)n.f0.choice;
        seq.elementAt(0).accept(this);
        AbstractExpression tmp = (AbstractExpression)nodeStack.pop();

        if(((NodeOptional)seq.elementAt(1)).present()){
          NodeOptional opt = (NodeOptional)seq.elementAt(1);

          if(((NodeChoice)opt.node).which == 0){
            tmp = tmp.postfixAddition();
            opt.accept(this);
          }
          else if(((NodeChoice)opt.node).which == 1){
            tmp = tmp.postfixSubtraction();
            opt.accept(this);
          }
          else{
            status.push(RECURSIVE);
            opt.accept(this);
            status.pop();

            switch( op ){

              case 0:
                tmp = tmp.assignment((AbstractExpression)nodeStack.pop());
                break;

              case 1:
                tmp = tmp.multiplicationAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 2:
                tmp = tmp.divisionAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 3:
                tmp = tmp.modulusAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 4:
                tmp = tmp.additionAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 5:
                tmp = tmp.subtractionAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 6:
                tmp = tmp.leftShiftAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 7:
                tmp = tmp.rightShiftAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 8:
                tmp = tmp.extensionRightShiftAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 9:
                tmp = tmp.andAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 10:
                tmp = tmp.exclusiveOrAssignment((AbstractExpression)nodeStack.pop());
                break;

              case 11:
                tmp = tmp.inclusiveOrAssignment((AbstractExpression)nodeStack.pop());
                break;

              default:
                break;
            }


          }

        }

        if (status.peek()==DEFAULT) { 
          refHashtable.put(n, tmp); 
        } 
        else if (status.peek()==RECURSIVE) { 
          nodeStack.push(tmp); 
        } 
        else { 
          throw new Error("Invalid status."); 
        } 
 
      } 
 
   }

}
