/*
 * File         : PrimaryExpressionManager.java
 * Author       : Guansong Zhang
 * Created      : Wed Aug 12 13:03:10 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1999/03/16 20:24:19 $
 */


package hpjava.compiler.abstractexp;

import java.io.*;
import java.util.*;

import hpjava.compiler.util.*;

import hpjava.compiler.parser.*;
import hpjava.compiler.unparser.*;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

import hpjava.compiler.reflection.*;

public class PrimaryExpressionManager {
  static final int PRIMNUMBER  = 1024;

  static Hashtable table = new Hashtable(PRIMNUMBER);

  public PrimaryExpressionManager(Node r){
    PrimaryExpressionConvertor pec = new PrimaryExpressionConvertor(table);
    r.accept(pec);
    
    if (false) { //debug
      System.err.println("Primary Expression: "+table+" created.");
      System.err.println("Primary Expression table created.");
      System.err.println();
    }
  }

  public PrimaryExpressionManager (Project prj) {
    PrimaryExpressionConvertor convertor = 
      new PrimaryExpressionConvertor(table);

    Enumeration enum = prj.rootFileTable.keys();

  LOOP: while (enum.hasMoreElements()) {
      Node root = (Node) enum.nextElement();
      root.accept(convertor);

    }

    if (prj.isVerbose()) {
      //System.err.println("Node scope: "+nodeScopeTable+" created.");
      System.err.println("Primary Expression table created.");
      System.err.println();
    }

  }

  static public AbstractPrimaryExpression lookup(PrimaryExpression n) {
    AbstractPrimaryExpression node = 
      (AbstractPrimaryExpression)table.get(n); 
      
    if (node != null)  
      return node;
    else {
      System.err.println("Not found primary expression:");
      n.accept(new TreePrinter());
      throw new Error();
    }
  }

  public static void register(Node n, AbstractPrimaryExpression exp) {
    table.put(n, exp);
    exp.setRegistration(n);
  }

  public void typeChecking() {
    if (!table.isEmpty()) {
      Enumeration enum = table.elements();

    LOOP:while (enum.hasMoreElements()) {

        AbstractPrimaryExpression node = 
          (AbstractPrimaryExpression) enum.nextElement();

        node.typeChecking();

      }

    }
  }
}

// The Convertor also needs to recursively 
// find out all the PrimaryExpression(s)
// seperate it as two? finder and convertor? 

class PrimaryExpressionConvertor extends ExpressionFindingVisitor {

  boolean debug=false;

  final int DEFAULT = 0;
  final int TYPEDEC = 1;

  private IntStack status=new IntStack();
  
  private Stack abstractNodeStack = new Stack();
  //private AbstractPrimaryExpression primary;

  protected Hashtable table;  // input;

  public PrimaryExpressionConvertor() {
    table = new Hashtable ();
    status.push(DEFAULT);

    debug=true;
  }

  public PrimaryExpressionConvertor(Hashtable table) {
    this.table = table;
    status.push(DEFAULT);
  }

   //
   // f0 -> PrimaryPrefix()
   // f1 -> ( PrimarySuffix() )*
   //
  public void visit(PrimaryExpression n) {
    //n.accept(new TreePrinter(System.out));

    n.f0.accept(this);

    if (n.f1.present()) {
      AbstractPrimaryExpression primary = 
        (AbstractPrimaryExpression) abstractNodeStack.pop();

      for(int i=n.f1.size()-1; i>=0;i--){
        n.f1.elementAt(i).accept(this);
      }
        
      for(int i=0; i<n.f1.size(); i++){        
        primary=primary.append((PrimaryItem) abstractNodeStack.pop());
      }

      abstractNodeStack.push(primary);
    }
     
    AbstractPrimaryExpression primary = 
      (AbstractPrimaryExpression) abstractNodeStack.pop();

    PrimaryExpressionManager.register(n, primary);

  }

   //
   // f0 -> Literal()
   //       | ResultType() "." "class"
   //       | AllocationExpression()
   //       | "this"
   //       | "super"
   //       | "(" Expression() ")"
   //       | <IDENTIFIER>
   //
   public void visit(PrimaryPrefix n) {
      Node repNode;
      PrimaryItem primary;
      
      switch (n.f0.which) {
      case 0:
        primary = new LiteralDescriptor((Literal)n.f0.choice);        
        break;
      case 1:
        repNode = ((NodeSequence) n.f0.choice).elementAt(0);
        primary = new ClassLiteralDescriptor((ResultType) repNode);
        break;
      case 2:
        primary = new AllocationExpressionDescriptor(
          (AllocationExpression)n.f0.choice);
        break;
      case 3:
        primary = new ThisDescriptor();
        break;
      case 4:
        primary = new SuperDescriptor();
        break;
      case 5:
        repNode = ((NodeSequence) n.f0.choice).elementAt(1);
        primary = new ExpressionDescriptor((Expression) repNode);
        break;
      case 6:
        primary = new IdentifierDescriptor((NodeToken)n.f0.choice);;
        break;
      default:
        throw new Error("Unexpected case.");
      }

      n.f0.accept(this);
      
      abstractNodeStack.push(primary);
      
   }

   //
   // f0 -> "." "this"
   //       | "." AllocationExpression()
   //       | "[" "[" DistArraySecRefDimList() "]" "]"
   //       | "[" TripletOrExpressionList() "]"
   //       | "." <IDENTIFIER>
   //       | Arguments()
   //
   public void visit(PrimarySuffix n) {
      PrimaryItem suffix = null;
      Node repNode = null;

      switch (n.f0.which) {
      case 0:
        repNode = ((NodeSequence) n.f0.choice).elementAt(1);
        suffix = new ThisDescriptor();
        break;
      case 1:
        repNode = ((NodeSequence) n.f0.choice).elementAt(1);
        suffix = new AllocationExpressionDescriptor
          ((AllocationExpression) repNode);
        break;
      case 2:
        repNode = ((NodeSequence) n.f0.choice).elementAt(2);
        suffix = new DistArraySecRefDimListDescriptor
          ((DistArraySecRefDimList) repNode);
        break;
      case 3:
        repNode = ((NodeSequence) n.f0.choice).elementAt(1);
        suffix = new TripletOrExpressionListDescriptor(
          (TripletOrExpressionList) repNode);
        break;
      case 4:
        repNode = ((NodeSequence) n.f0.choice).elementAt(1);
        suffix = new IdentifierDescriptor((NodeToken) repNode);
        break;
      case 5:
        repNode = (Arguments) n.f0.choice;
        suffix = new ArgumentsDescriptor((Arguments) repNode);
        break;
      default:
        throw new Error("Unexpected case.");
      }

      abstractNodeStack.push(suffix);

      n.f0.accept(this);
      //      System.out.println("__________"+primary.toString());
   }

}
