/*
 * File         : TripletOrExpressionList.java
 * Author       : Guansong Zhang
 * Created      : Wed Aug 12 14:23:07 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1999/03/16 20:24:29 $
 */

package hpjava.compiler.abstractexp;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

public class TripletOrExpressionListDescriptor extends PrimaryItem {
  public TripletOrExpressionList leaf;

  public TripletOrExpressionListDescriptor (TripletOrExpressionList leaf) {
    this.leaf = leaf;
  }

  public TripletOrExpressionListDescriptor (Expression exp) {
    NodeChoice choic = new NodeChoice (exp,1);

    TripletOrExpression toe = new TripletOrExpression(choic);
    NodeListOptional listOptional = new NodeListOptional();
    leaf = new TripletOrExpressionList(toe, listOptional);

  }

  public TripletOrExpressionListDescriptor (AbstractExpression exp) {
    Expression empty = new Expression(null, null);
    ExpressionManager.register(empty, exp);

    NodeChoice choic = new NodeChoice (empty,1);

    TripletOrExpression toe = new TripletOrExpression(choic);
    NodeListOptional listOptional = new NodeListOptional();
    leaf = new TripletOrExpressionList(toe, listOptional);
  }

  public TripletOrExpressionListDescriptor (AbstractPrimaryExpression exp) {
    PrimaryExpression emptyPrimary = new PrimaryExpression(null, null);
    PrimaryExpressionManager.register(emptyPrimary, exp);

    AbstractExpression aexp = new PrimaryExpressionNode(emptyPrimary);

    Expression empty = new Expression(null, null); 
    ExpressionManager.register(empty, aexp); 
  
    NodeChoice choic = new NodeChoice (empty,1); 

    TripletOrExpression toe = new TripletOrExpression(choic); 
    NodeListOptional listOptional = new NodeListOptional(); 
    leaf = new TripletOrExpressionList(toe, listOptional); 

  }

  public void accept(AbstractPrimaryExpressionVisitor v) {
    v.visit(this);
  }

  public int numberOfDims() {
   //
   // f0 -> TripletOrExpression()
   // f1 -> ( "," TripletOrExpression() )*
   //
    TripletOrExpressionList list = this.leaf;
    
    if (list.f1.present()) {
      return 1+( (NodeListOptional)list.f1 ).size();
    }
    else {
      return 1;
    }    
  }

  public DimensionDescriptor dim(int i) {
    if (i==0) {
      TripletOrExpression toe = this.leaf.f0;

      if (toe.f0.which==0) {
        return new TripletDescriptor((Triplet)toe.f0.choice);
      }
      else if (toe.f0.which==1) {
        return new ExpressionDescriptor((Expression)toe.f0.choice);
      }
      else {
        throw new Error();
      }
    }
    else {
      NodeSequence sequence = (NodeSequence)this.leaf.f1.elementAt(i-1);
      TripletOrExpression toe = (TripletOrExpression)sequence.elementAt(1);

      if (toe.f0.which==0) {
        return new TripletDescriptor((Triplet)toe.f0.choice);
      }
      else if (toe.f0.which==1) {
        return new ExpressionDescriptor((Expression)toe.f0.choice);
      }
      else {
        throw new Error();
      }
    }
  }

  public boolean isExpressionDim(int i) {
    if (i==0) {
      TripletOrExpression toe = this.leaf.f0;

      if (toe.f0.which==0) {
        return false;
      }
      else if (toe.f0.which==1) {
        return true;
      }
      else {
        throw new Error();
      }
    }
    else {
      NodeSequence sequence = (NodeSequence)this.leaf.f1.elementAt(i-1);
      TripletOrExpression toe = (TripletOrExpression)sequence.elementAt(1);

      if (toe.f0.which==0) {
        return false;
      }
      else if (toe.f0.which==1) {
        return true;
      }
      else {
        throw new Error();
      }
    }
  }

  public boolean isTripletDim(int i) {
    if (i==0) {
      TripletOrExpression toe = this.leaf.f0;

      if (toe.f0.which==0) {
        return true;
      }
      else if (toe.f0.which==1) {
        return false;
      }
      else {
        throw new Error();
      }
    }
    else {
      NodeSequence sequence = (NodeSequence)this.leaf.f1.elementAt(i-1);
      TripletOrExpression toe = (TripletOrExpression)sequence.elementAt(1);

      if (toe.f0.which==0) {
        return true;
      }
      else if (toe.f0.which==1) {
        return false;
      }
      else {
        throw new Error();
      }
    }
  }


}
