/*
 * File         : ClassReflector.java
 * Author       : Guansong Zhang
 * Created      : Mon May 18 17:20:06 1998
 * Revision     : $Revision: 1.5 $
 * Updated      : $Date: 1998/10/07 22:56:08 $
 */

package hpjava.compiler.reflection;

import java.io.*;
import java.lang.reflect.*;

abstract public class ClassReflector {

  //abstract public ClassReflector getDeclaringClass();

  abstract public ClassReflector getComponentType();

  abstract public ConstructorReflector getConstructor(String [] paratypeName)
    throws NoSuchMethodException, SecurityException ;

  abstract public ConstructorReflector[] getConstructors()
    throws SecurityException ;

  abstract public FieldReflector getDeclaredField(String name)
    throws NoSuchFieldException, SecurityException ;

  abstract public FieldReflector[] getDeclaredFields()
    throws SecurityException ;

  abstract public MethodReflector getDeclaredMethod(String name, String [] paratypeName) 
    throws NoSuchMethodException, SecurityException ;

  abstract public MethodReflector[] getDeclaredMethods()
    throws SecurityException ;

  abstract public String getSuperclassName();

  abstract public String[] getInterfaceNames();

  abstract public ModifierReflector getModifier();

  abstract public String getName();

  abstract public boolean isArray();

  abstract public boolean isDistArray();
  
  abstract public boolean isInterface();

  abstract public boolean isPrimitive();

  abstract protected void showConstructors(String space, PrintStream out);
  abstract protected void showDeclaredFields(String space, PrintStream out);
  abstract protected void showDeclaredMethods(String space, PrintStream out);

  public void show(PrintStream out) {
    out.println("CLASS:");    
    
    if (isInterface())
      out.println("  NAME(I):\t"+ getName());
    else
      out.println("  NAME(C):\t"+ getName());

    ModifierReflector mr=getModifier();
    if(mr!=null)
      mr.show("  ", out);
      
    if (getSuperclassName()!=null)
      out.println("  SUPERCLASS:\t"    + getSuperclassName());

    String [] InterfaceNames = getInterfaceNames();
    if(InterfaceNames!=null) {
      out.print("  INTERFACES:\t") ;
      for (int i=0; i<InterfaceNames.length; i++) 
        out.print(InterfaceNames[i]+" ");
      out.println();  
    }          

    showConstructors("  ", out);
    showDeclaredFields("  ",out);
    showDeclaredMethods("  ", out);
  } 

}
