/*
 * File         : ClassReflectorByByteCode.java
 * Author       : Guansong Zhang
 * Created      : Fri May 22 11:23:17 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/10/08 04:00:19 $
 */

package hpjava.compiler.reflection;

import java.io.*;
import java.lang.reflect.*;

public class ClassReflectorByByteCode extends ClassReflector {

  protected Class cls;

  // a public method for creating reflector from Java reflection.

  public ClassReflectorByByteCode (Class cls) {
    this.cls=cls;
  }

  public ClassReflector getComponentType() {
    return new ClassReflectorByByteCode(cls.getComponentType());
  }


  public ConstructorReflector getConstructor(String [] paratypeName)
    throws NoSuchMethodException, SecurityException {

      Class [] paratype = null;

      if (paratypeName != null) {
  paratype = new Class [paratypeName.length];

  try {
    for (int i = 0; i< paratypeName.length; i++)
      paratype[i] = Clazz.forName(paratypeName[i]);
  }
  catch (ClassNotFoundException e) {
    // Make sure the class for the name already found? performance
    System.err.println(e.getMessage());
  }
      }
      else
  paratype = null;

      return ( new ConstructorReflectorByteCode (cls.getConstructor(paratype)) );
  }

  public ConstructorReflector[] getConstructors()
    throws SecurityException {
      Constructor [] cList = cls.getConstructors();
      if (cList == null)
  return null;
      else {
  ConstructorReflector [] crList = new ConstructorReflector[cList.length];
  for (int i=0; i<crList.length; i++)
    crList[i]= new ConstructorReflectorByteCode(cList[i]);
  return crList;
      }
  }

  public FieldReflector getDeclaredField(String name)
    throws NoSuchFieldException, SecurityException  {
      return (new FieldReflectorByteCode(cls.getDeclaredField(name)));
  }

  public FieldReflector[] getDeclaredFields()
    throws SecurityException {
      Field [] fList = cls.getDeclaredFields();
      if (fList == null)
  return null;
      else {
  FieldReflector [] frList = new FieldReflector[fList.length];
  for (int i=0; i<frList.length; i++)
    frList[i]= new FieldReflectorByteCode(fList[i]);
  return frList;
      }
  }

  public MethodReflector getDeclaredMethod(String name, String [] paratypeName)
    throws NoSuchMethodException, SecurityException {

      Class [] paratype = null;

      if (paratypeName != null) {
  paratype = new Class [paratypeName.length];

  try {
    for (int i = 0; i< paratypeName.length; i++)
      paratype[i] = Clazz.forName(paratypeName[i]);
  }
  catch (ClassNotFoundException e) {
    // Make sure the class for the name already found? performance
    System.err.println(e.getMessage());
  }
      }
      else
  paratype = null;

      return ( new MethodReflectorByteCode (cls.getDeclaredMethod(name, paratype)) );
  }

  public MethodReflector[] getDeclaredMethods()
    throws SecurityException {
      Method [] mList = cls.getDeclaredMethods();
      if (mList == null)
  return null;
      else {
  MethodReflector [] mrList = new MethodReflector[mList.length];
  for (int i=0; i<mrList.length; i++)
    mrList[i]= new MethodReflectorByteCode(mList[i]);
  return mrList;
      }
  }

  public String getSuperclassName() {
    return cls.getSuperclass().getName();
  }

  public String[] getInterfaceNames() {
    Class [] interfaces=cls.getInterfaces();

    String [] names;

    if (interfaces == null)
      names=null;
    else {
      names=new String[interfaces.length];
      for (int i=0; i<interfaces.length; i++)
  names[i]=interfaces[i].getName();
    }

    return names;
  }

  public ModifierReflector getModifier() {
    return new ModifierReflector(this.cls.getModifiers());
  }

  public String getName() {
    return this.cls.getName();
  }

  public boolean isArray() {
    return cls.isArray();
  }

  public boolean isDistArray() {
    if (this.cls.getName().startsWith("hpjava.lang.Array"))
      return true;

    if (this.cls.getName().startsWith("hpjava.lang.Section"))
      return true;

    return false;
  }

  public boolean isInterface() {
    return cls.isInterface();
  }

  public boolean isPrimitive() {
    return cls.isPrimitive();
  }

  protected void showConstructors(String space, PrintStream out) {
    ConstructorReflector [] crList = getConstructors();
    if (crList!=null) {
      for (int i=0; i<crList.length; i++)
  crList[i].show(space, out);
    }

  }

  protected void showDeclaredFields(String space, PrintStream out) {

  }

  protected void showDeclaredMethods(String space, PrintStream out) {

  }
}
