/*
 * File         : ClassReflectorBySourceCode.java
 * Author       : Guansong Zhang
 * Created      : Tue May 19 11:29:22 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1999/03/16 20:27:29 $
 */

package hpjava.compiler.reflection;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;

import hpjava.compiler.util.*; 

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

public class ClassReflectorBySourceCode extends ClassReflector {

  public ClassReflector getComponentType() {
    return null;
  }

  protected Hashtable constructors;

  public ConstructorReflector getConstructor(String [] paratypeName) 
    throws NoSuchMethodException {
      StringBuffer signature = new StringBuffer("<init>");
      
      if (paratypeName != null)
      	for (int i=0; i<paratypeName.length; i++) 
	  signature.append(paratypeName[i]);

      ConstructorReflector m = (ConstructorReflector) constructors.get(signature.toString());
      
      if (m == null)
      	throw new NoSuchMethodException (signature.toString());
      else 
	return m;
  }

  public ConstructorReflector[] getConstructors() {
    if(constructors == null )
      return null;
    else {
      int size = constructors.size();
      ConstructorReflector[] crList = new ConstructorReflector[size];
      Enumeration enu = constructors.elements();
      for (int i=0; i<size; i++)
	crList[i]=(ConstructorReflector)enu.nextElement();
      return crList;
    }
  }

  protected Hashtable fields;

  public FieldReflector getDeclaredField(String name)
    throws NoSuchFieldException {
      FieldReflector f = (FieldReflector) fields.get(name);
      
      if (f == null)
	throw new NoSuchFieldException (name);
      else
	return f;
  }

  public FieldReflector[] getDeclaredFields() {
    if(fields == null)
      return null;
    else {
      int size = fields.size();
      FieldReflector[] frList = new FieldReflector[size];
      Enumeration enu = fields.elements();
      for (int i=0; i<size; i++)
	frList[i]=(FieldReflector)enu.nextElement();
      return frList;
    }
  }

  protected Hashtable methods;

  public MethodReflector getDeclaredMethod(String name, String [] paratypeName) 
    throws NoSuchMethodException {
      StringBuffer signature = new StringBuffer(name);
      
      if (paratypeName!=null)
      	for (int i=0; i<paratypeName.length; i++) 
	  signature.append(paratypeName[i]);

      MethodReflector m = (MethodReflector) methods.get(signature.toString());
      
      if (m == null)
      	throw new NoSuchMethodException (signature.toString());
      else
	return m;
  }

  public MethodReflector[] getDeclaredMethods() {
    if(methods == null)
      return null;
    else {
      int size = methods.size();
      MethodReflector[] mrList = new MethodReflector[size];
      Enumeration enu = methods.elements();
      for (int i=0; i<size; i++)
	mrList[i]=(MethodReflector)enu.nextElement();
      return mrList;
    }
  }

  protected Hashtable innerclass;

  protected String superClassName ;

  public String getSuperclassName() {
    return superClassName;
  }

  protected String[] interfaceNames ;
  
  public String[] getInterfaceNames() {
    return interfaceNames;
  }
  
  protected ModifierReflector modifier;
  
  public ModifierReflector getModifier() {
    return modifier;
  }

  protected String name="<null>";

  public String getName() {
    return name;
  }

  protected boolean _array = false;

  public boolean isArray() {
    return _array;
  }

  protected boolean _distarray = false;

  public boolean isDistArray() {
    return _distarray;
  }

  protected boolean _interface;

  public boolean isInterface() {
    return _interface;
  }

  protected boolean _primitive;

  public boolean isPrimitive() {
    return _primitive;
  }

  protected void showConstructors(String space, PrintStream out) {
    if (constructors!=null) {
      Enumeration enu = constructors.keys();
      while (enu.hasMoreElements()) {
	String signature = (String)enu.nextElement();
	out.println(space+"SIGNATURE(c):\t"+signature);
	ConstructorReflector cr = (ConstructorReflector) constructors.get(signature);
	cr.show(space, out);
      }
    }
  }

  protected void showDeclaredFields(String space, PrintStream out) {
    if (fields!=null) {
      Enumeration enu = fields.keys();
      while (enu.hasMoreElements()) {
	String signature = (String)enu.nextElement();
	out.println(space+"SIGNATURE(f):\t"+signature);
	FieldReflector fr = (FieldReflector) fields.get(signature);
	fr.show(space, out);
      }
    }
  }

  protected void showDeclaredMethods(String space, PrintStream out) {
    if (methods!=null) {
      Enumeration enu = methods.keys();
      while (enu.hasMoreElements()) {
	String signature = (String)enu.nextElement();
	out.println(space+"SIGNATURE(m):\t"+signature);
	MethodReflector mr = (MethodReflector) methods.get(signature);
	mr.show(space, out);
      }
    }
  }

  // protected field can be accessed by Class in the same Package!

  public ClassReflectorBySourceCode(String pname, Vector inames, Node n) {
    ClassReflectorComposer composer = new ClassReflectorComposer(pname, inames, this);
    n.accept(composer); 
  }  

}

class ClassReflectorComposer extends RecoveryVisitor {

  boolean debug=false;

  final int DEFAULT    = 0;
  final int SUPERCLASS = 1;
  final int INTERFACE  = 2; 

  private IntStack status=new IntStack();

  private String pname;
  private Vector inames;
  
  private StringBuffer nameBuffer;
  
  private ClassReflectorBySourceCode clsr;   

  private Vector interfaceNames;

  // for multi var in a field
  private ModifierReflector mod;
  private ArraySignaturesImage prefix;

  private ScalarReflectorDescriptor srd;

  public ClassReflectorComposer (String pname, Vector inames, ClassReflectorBySourceCode clsr) {
    this.pname=pname;
    this.inames=inames;
    this.clsr=clsr;

    nameBuffer = new StringBuffer(pname);    

    status.push(DEFAULT);  
  }


   //
   // f0 -> ( "abstract"| "final" | "public"  )*
   // f1 -> UnmodifiedClassDeclaration()
   //
   public void visit(ClassDeclaration n) {
      if ( n.f0.present() ) {
        ModifierReflector mr = new ModifierReflector(n.f0);
        
        clsr.modifier = mr;
      }
      n.f1.accept(this);
   }

   //
   // f0 -> ( "static" | "abstract" | "final" | "public" | "protected" | "private" )*
   // f1 -> UnmodifiedClassDeclaration()
   //
   public void visit(NestedClassDeclaration n) {
      if ( n.f0.present() ) {
        ModifierReflector mr = new ModifierReflector(n.f0);
        clsr.modifier = mr;
      }
      n.f1.accept(this);
   }

  /*
    This is used by ClassDeclaration, NestedClassDeclaration,
    and localClassDeclaration (UnmodifiedClassDeclaration directly)
  */
   //
   // f0 -> "class"
   // f1 -> < IDENTIFIER >
   // f2 -> [ "extends" Name() ]
   // f3 -> [ "implements" NameList() ]
   // f4 -> ClassBody()
   //
   // continued from ClassDeclaration
   //
   public void visit(UnmodifiedClassDeclaration n) {
   
      ClassTypeDescriptor cnd = 
          new ClassTypeDescriptor(pname, inames, n.f1);

      clsr.name=cnd.name;

      if ( n.f2.present() ) {
        status.push(SUPERCLASS);
        visit (n.f2);
        status.pop();        
      }
      
      if ( n.f3.present() ) { 
        status.push(INTERFACE);
        visit (n.f3);
        status.pop();        
        
      }
      // this lead to ClassBody().
      n.f4.accept(this);
   }
   

   //
   // f0 -> ( "abstract" | "public" )*
   // f1 -> UnmodifiedInterfaceDeclaration()
   //
   public void visit(InterfaceDeclaration n) {
      clsr._interface = true;
      if ( n.f0.present() ) {
        ModifierReflector mr = new ModifierReflector(n.f0);
        mr._interface=true;
        clsr.modifier = mr;
      }
      n.f1.accept(this);
   }

   //
   // f0 -> ( "static" | "abstract" | "final" | "public" | "protected" | "private" )*
   // f1 -> UnmodifiedInterfaceDeclaration()
   //
   public void visit(NestedInterfaceDeclaration n) {
      clsr._interface = true;
      if ( n.f0.present() ) {
        ModifierReflector mr = new ModifierReflector(n.f0);
        mr._interface=true;
        clsr.modifier = mr;
      }
      n.f1.accept(this);
   }


  /*
    This is used by InterfaceDeclaration, NestedInterfaceDeclaration,
    and localInterfaceDeclaration (UnmodifiedInterfaceDeclaration directly)
  */  
   //
   // f0 -> "interface"
   // f1 -> <IDENTIFIER>
   // f2 -> [ "extends" NameList() ]
   // f3 -> "{"
   // f4 -> ( InterfaceMemberDeclaration() )*
   // f5 -> "}"
   //
   public void visit(UnmodifiedInterfaceDeclaration n) {
            
      ClassTypeDescriptor cnd = 
          new ClassTypeDescriptor(pname, inames, n.f1);

      clsr.name=cnd.name;

      status.push(INTERFACE);
      n.f2.accept(this);
      status.pop();
    
      //skip nested declaration, find members only.      
      n.f4.accept(this);
   }


   //
   // f0 -> NestedClassDeclaration()
   //       | NestedInterfaceDeclaration()
   //       | MethodDeclaration()
   //       | FieldDeclaration()
   //       | ";"
   //
   public void visit(InterfaceMemberDeclaration n) {
     switch (n.f0.which) {
     case 2:
     case 3:
       n.f0.accept(this);
       break;
     default:
       break;
     }
   }


   //
   // f0 -> "{"
   // f1 -> ( ClassBodyDeclaration() )*
   // f2 -> "}"
   //
   public void visit(ClassBody n) {     
      recoveryVisit(n.f1);
   }


   //
   // f0 -> Initializer()
   //       | NestedClassDeclaration()
   //       | NestedInterfaceDeclaration()
   //       | ConstructorDeclaration()
   //       | MethodDeclaration()
   //       | FieldDeclaration()
   //       | ";"
   //
   public void visit(ClassBodyDeclaration n) {
     //only take care of members   
     switch (n.f0.which) {
     case 3:
     case 4:
     case 5:
       n.f0.accept(this);
       break;
     default:
       break;
     }
   }

   //
   // f0 -> [ "public" | "protected" | "private" ]
   // f1 -> <IDENTIFIER>
   // f2 -> FormalParameters()
   // f3 -> [ "throws" NameList() ]
   // f4 -> "{"
   // f5 -> [ ExplicitConstructorInvocation() ]
   // f6 -> ( BlockStatement() )*
   // f7 -> "}"
   //
   public void visit(ConstructorDeclaration n) {
      clsr.constructors = new Hashtable();
   
      if (!clsr.name.endsWith(n.f1.tokenImage)) {
        String message = "constructor "+ n.f1.tokenImage +
	  " should have the same name as the class or there is no return type for this methord.";

        throw new CompilingError(message, n.f1);            
      }
      
      ConstructorReflectorSourceCode conr = new ConstructorReflectorSourceCode(pname, inames, n);
      conr.declaringClassReflector=clsr;
      
      MemberSignatureImage md= new MemberSignatureImage(pname, inames, n);
         
      clsr.constructors.put(md.name, conr);
   }


  //the following two may also come from an interface;

   //
   // f0 -> ( "public" | "protected" | "private" | "static" | "abstract" | "final" | "native" | "synchronized" )*
   // f1 -> ResultType()
   // f2 -> MethodDeclarator()
   // f3 -> [ "throws" NameList() ]
   // f4 -> ( Block() | ";" )
   //
   public void visit(MethodDeclaration n) {
      clsr.methods = new Hashtable();

      MethodReflectorSourceCode mr = 
	new MethodReflectorSourceCode(pname, inames, n);

      mr.declaringClassReflector=clsr;
   
      MemberSignatureImage md = 
	new MemberSignatureImage(pname, inames, n);

      clsr.methods.put(md.name, mr);
   }

   //
   // f0 -> ( "public" | "protected" | "private" | "static" | "final" | "transient" | "volatile" )*
   // f1 -> Type()
   // f2 -> VariableDeclarator()
   // f3 -> ( "," VariableDeclarator() )*
   // f4 -> ";"
   //
   public void visit(FieldDeclaration n) {
     clsr.fields = new Hashtable();

     // possible multi var in a field, so it is different from constructor and method

     if (n.f0.present()) 
       this.mod = new ModifierReflector(n.f0);

     //
     // f0 -> ( PrimitiveType() | Name() )
     // f1 -> ( ArraySignature() )*
     //
     //public void visit(Type n) 

     if (n.f1.f1.present())
       this.prefix = new ArraySignaturesImage(n.f1.f1);

     switch (n.f1.f0.which) { 
     case 0:
       this.srd = new PrimitiveTypeDescriptor((PrimitiveType)n.f1.f0.choice);
       break;
     case 1:
       this.srd = new ClassTypeDescriptor(pname, inames, (Name)n.f1.f0.choice); 
       break;  
     default:
       throw new Error();
     }
     
     n.f2.accept(this);
     if (n.f3.present())
       n.f3.accept(this);
   }


   //
   // f0 -> VariableDeclaratorId()
   // f1 -> [ "=" VariableInitializer() ]
   //

   public void visit(VariableDeclarator n) {
      n.f0.accept(this);
   }

   //
   // f0 -> <IDENTIFIER>
   // f1 -> ( ArraySignature() )*
   //
   public void visit(VariableDeclaratorId n) {
     FieldReflectorSourceCode fr = 
	new FieldReflectorSourceCode(this.srd, this.prefix, n);

      fr.modifier = this.mod;
      fr.declaringClassReflector=clsr;

      clsr.fields.put(n.f0.tokenImage, fr);

   }


   //
   // f0 -> Name()
   // f1 -> ( "," Name() )*
   //
   public void visit(NameList n) {
      switch (status.peek()) {
      case INTERFACE:
        // create different interfaces. 
        
        interfaceNames = new Vector();
        
        n.f0.accept(this);        
        if (n.f1.present()) {
          n.f1.accept(this);        
        }
        
        String[] interfaceNameList = new String [interfaceNames.size()];

        Enumeration e = interfaceNames.elements();
        for (int i = 0; i< interfaceNameList.length; i++)
          interfaceNameList[i]=(String) e.nextElement();
          
        clsr.interfaceNames = interfaceNameList;
          
        break;
                    
      default:
        break;
      }
    }

   //
   // f0 -> < IDENTIFIER >
   // f1 -> ( "." < IDENTIFIER >  )*
   //
   public void visit (Name n) {
      switch (status.peek()) {
      case SUPERCLASS: 
      case INTERFACE: {
        ClassTypeDescriptor cn = 
          new ClassTypeDescriptor(pname, inames, n);
        
        if (status.peek() == SUPERCLASS)
          clsr.superClassName=cn.name;
          
        if (status.peek() == INTERFACE)  
          interfaceNames.addElement(cn.name);
          
        break;
      }    

      default:
        break;
      }
   }

}

// The finder is responsible for finding inner class. 
// The composer inside ClassReflectorBySourceCode constructor only fills the reflector.
// This includes constructors, fields, methods and possibly member class name


/* To do list

a way to construct compound class for distributed array is needed. 

*/
