/*
 * File         : AbstractPrimaryExpressionTranslator.java
 * Author       : Guansong Zhang
 * Created      : Thu Oct  8 12:41:06 1998
 * Revision     : $Revision: 1.14 $
 * Updated      : $Date: 1999/03/16 20:29:44 $
 */


package hpjava.compiler.translator;

import hpjava.compiler.util.*;

import hpjava.compiler.syntaxtree.*;

import hpjava.compiler.unparser.*;

import hpjava.compiler.abstractexp.*;
import hpjava.compiler.reflection.*;


public class AbstractPrimaryExpressionTranslator {
  AbstractPrimaryExpression exp;

  public AbstractPrimaryExpressionTranslator(AbstractPrimaryExpression exp) {
    this.exp = exp; 
  }
  
  public void process() {
    AbstractPrimaryExpressionProcessor processor = 
      new AbstractPrimaryExpressionProcessor();

    this.exp.accept(processor);
  }
  
}

class AbstractPrimaryExpressionProcessor 
  extends AbstractPrimaryExpressionVisitor 
{

  final int PREFIX = 0;
  final int SUFFIX = 1;

  private ClassReflector cr;
  private AbstractPrimaryExpression prefix;

  private IntStack status=new IntStack();

  private StringBuffer image = new StringBuffer();

  public AbstractPrimaryExpressionProcessor () {
    status.push(PREFIX);
    // for simple expression with out concatenation, it is a prefix.
  }


  // begin for TripletOrExpressionListDescriptor
  private void toeldInSuffix (PrimaryConcatenation n) {
    if (this.cr.isDistArray()) { 

      TripletOrExpressionListDescriptor suffix = 
        (TripletOrExpressionListDescriptor)n.suffix;

      AbstractPrimaryExpression data;

      data = this.prefix.append(new IdentifierDescriptor("dat"));
      data = data.append(new ArgumentsDescriptor());

      AbstractPrimaryExpression position = 
        this.prefix.append(new IdentifierDescriptor("pos"));

      ArgumentsDescriptor posArg = new ArgumentsDescriptor();

      for (int i = 0; i < suffix.numberOfDims(); i++) {
        ExpressionDescriptor ed = (ExpressionDescriptor)suffix.dim(i);
        posArg.addArg(ed.leaf);
      }

      position = position.append (posArg);

      data = data.append(new TripletOrExpressionListDescriptor(position));

      n.replaceWith(data);

    } else {
      // if this is a range.
    }
  }
  // end for TripletOrExpressionListDescriptor 


  // begin for DistArraySecRefDimListDescriptor 

  private void addSecGroup(PrimaryConcatenation n, 
                           ArgumentsDescriptor arg) {

    DistArraySecRefDimListDescriptor dasrd = 
      (DistArraySecRefDimListDescriptor)n.suffix; 

    AbstractPrimaryExpression group = 
      n.prefix.append(new IdentifierDescriptor("grp"));
    
    group=group.append(new ArgumentsDescriptor());


    for (int i =0; i<dasrd.numberOfDims(); i++) { 
      if (dasrd.isExpressionDim(i)) { 
        group=group.append(new IdentifierDescriptor("divide"));

       group= group.append
         (new ArgumentsDescriptor(((ExpressionDescriptor)dasrd.dim(i)).leaf));
      } 
 
    } 
    
    arg.addArg(group);
  }

  private void addSecDimension(PrimaryConcatenation n, 
                               ArgumentsDescriptor arg) {

    DistArraySecRefDimListDescriptor dasrd =   
      (DistArraySecRefDimListDescriptor)n.suffix;   

    /*  
    AbstractPrimaryExpression range =   
      n.prefix.append(new IdentifierDescriptor("rng"));  

    range = range.append(new ArgumentsDescriptor());

    for (int i =0; i<dasrd.numberOfDims(); i++) {  
      if (!dasrd.isExpressionDim(i)) {  
        AbstractPrimaryExpression dim = new LiteralDescriptor(i);
        AbstractPrimaryExpression rng
          =range.append(new TripletOrExpressionListDescriptor(dim)); 

        arg.addArg(rng);

      }  
    }  
    
    AbstractPrimaryExpression stride =   
      n.prefix.append(new IdentifierDescriptor("str"));  

    stride = stride.append(new ArgumentsDescriptor());

    for (int i =0; i<dasrd.numberOfDims(); i++) {  
      if (!dasrd.isExpressionDim(i)) {  
        AbstractPrimaryExpression dim = new LiteralDescriptor(i);
        AbstractPrimaryExpression str
          =stride.append(new TripletOrExpressionListDescriptor(dim)); 

        arg.addArg(str);

      }  
    }  
    */

    AbstractPrimaryExpression range =   
      n.prefix.append(new IdentifierDescriptor("rng"));  

    range = range.append(new ArgumentsDescriptor());

    AbstractPrimaryExpression stride =   
      n.prefix.append(new IdentifierDescriptor("str"));  

    stride = stride.append(new ArgumentsDescriptor());

    for (int i =0; i<dasrd.numberOfDims(); i++) {  
      if (!dasrd.isExpressionDim(i)) {  // should be not a location expression
        AbstractPrimaryExpression dim = new LiteralDescriptor(i);

        AbstractPrimaryExpression rng
          =range.append(new TripletOrExpressionListDescriptor(dim)); 
        arg.addArg(rng);

        AbstractPrimaryExpression str
          =stride.append(new TripletOrExpressionListDescriptor(dim)); 
        arg.addArg(str);

      }  
    }  

    

  }

  private void addSecDate(PrimaryConcatenation n, 
                          ArgumentsDescriptor arg) {

    DistArraySecRefDimListDescriptor dasrd =  
      (DistArraySecRefDimListDescriptor)n.suffix;  
 
    AbstractPrimaryExpression date =  
      n.prefix.append(new IdentifierDescriptor("dat")); 

    date = date.append(new ArgumentsDescriptor());

    arg.addArg(date);

  }

  private void addSecOffset(PrimaryConcatenation n, ArgumentsDescriptor arg) {
    DistArraySecRefDimListDescriptor dasrd =   
      (DistArraySecRefDimListDescriptor)n.suffix;   

    AbstractPrimaryExpression parent = 
      n.prefix.append(new IdentifierDescriptor("off"));

    parent = parent.append(new ArgumentsDescriptor());
  
    AbstractExpression offset = parent.toAbstractExpression();
    
    IdentifierDescriptor primaryOffset =     
      new IdentifierDescriptor("offset");    

    AbstractPrimaryExpression stride =    
      n.prefix.append(new IdentifierDescriptor("str"));   
 
    for (int i =0; i<dasrd.numberOfDims(); i++) {   
      if (dasrd.isExpressionDim(i)) {   
        AbstractPrimaryExpression dim = new LiteralDescriptor(i);
         
        AbstractPrimaryExpression str = stride.append(new ArgumentsDescriptor());

        str = str.append(new TripletOrExpressionListDescriptor(dim));
        str = str.append(primaryOffset);
        str = str.append(
          new ArgumentsDescriptor( 
            ((ExpressionDescriptor)dasrd.dim(i)).leaf 
          )
        );

        if (offset == null) {
          offset = str.toAbstractExpression();
        }
        else {
          offset = offset.add(str.toAbstractExpression());
        }
 
      }   
    }   
 
    arg.addArg(offset); 

  }


  private void dasrdldInSuffix(PrimaryConcatenation n) {
    
    DistArraySecRefDimListDescriptor dasrd =
      (DistArraySecRefDimListDescriptor)n.suffix;

    ClassReflectorByDescriptor cr = 
      (ClassReflectorByDescriptor)n.prefix.getClassReflector();

    ClassReflector base = cr.getComponentType();

    if (!base.isPrimitive()) 
      throw new Error("Not finished");

    // the following line should be changed to deal with none primitive distarray.
    PrimitiveTypeDescriptor baseDescriptor = 
      new PrimitiveTypeDescriptor(base.getName());

    ArraySignaturesImage image = 
      new ArraySignaturesImage(dasrd.secRefSignatureOf(cr));
    
    ClassReflectorDescriptor crd = 
      new ArrayReflectorDescriptor(baseDescriptor,image.name);
    
    ClassReflector secReflector =
      new ClassReflectorByDescriptor(crd);

    ArgumentsDescriptor arg = new ArgumentsDescriptor();

    addSecGroup(n, arg);
    addSecDimension(n, arg);
    addSecDate(n, arg);
    addSecOffset(n, arg);

    AllocationExpressionDescriptor aed = 
      new AllocationExpressionDescriptor(secReflector);

    aed.addArguments(arg);

    n.replaceWith(aed);

  }
  // end for DistArraySecRefDimListDescriptor 


  public void visit (PrimaryConcatenation n) {
    status.push(PREFIX);
    n.prefix.accept(this);
    status.pop();

    this.prefix = n.prefix;

    if (n.suffix instanceof ThisDescriptor ) {
      // do nothing
    } 
    else if (n.suffix instanceof AllocationExpressionDescriptor ) { 
      // do nothing
    } 
    else if (n.suffix instanceof DistArraySecRefDimListDescriptor ) { 
      dasrdldInSuffix(n);
    } 
    else if (n.suffix instanceof TripletOrExpressionListDescriptor ) { 
      toeldInSuffix(n);
    } 
    else if (n.suffix instanceof IdentifierDescriptor ) { 
      // do nothing 
    } 
    else if (n.suffix instanceof ArgumentsDescriptor ) { 
      ArgumentsDescriptor args = (ArgumentsDescriptor) n.suffix;

      for (int i =0; i< args.numberOfArgs(); i++) {
        AbstractExpression aexp = args.abstractArg(i);

        AbstractExpressionTranslator translator 
          = new AbstractExpressionTranslator(aexp);

        translator.process();
      }
    } 
    else {
      throw new Error();
    }


  }

  // this is for a simple distributed array.
  // here is a temp one.
  // it is much more complicated if you provide a complete solution
  // the AllocationExpression can not be translated into a single expression.
  // such as multiple allocated shapes.
  private void aedAsPrefix (AllocationExpressionDescriptor n) {

    AllocationExpression alloc = n.leaf;

    AllocationExpressionDescriptor allocDescriptor
      = new AllocationExpressionDescriptor(this.cr);

    

    ArgumentsDescriptor args = new ArgumentsDescriptor();
    
    AbstractPrimaryExpression group = new IdentifierDescriptor("SpmdEnv");
    group = group.append(new IdentifierDescriptor("apg"));
    
    args.addArg(group);

    AllocatedShapeDescriptor descriptor = n.allocatedShape(0);

    for (int i =0; i< descriptor.numberOfDims(); i++) {
      args.addArg(descriptor.dim(i));
    }

    allocDescriptor.addArguments(args);

    n.replaceWith(allocDescriptor);

  }

  public void visit (AllocationExpressionDescriptor n) {
    if (status.peek()==PREFIX) {
      this.cr = n.getClassReflector();
      if (cr != null) { // not finished

        if (cr.isDistArray()) {
          aedAsPrefix(n);
        }

      }
    }
    else if (status.peek()==SUFFIX) {

    }
    else {
      throw new Error();
    }

  }

  public void visit (IdentifierDescriptor n) {
    if (status.peek()==PREFIX) {
      this.cr = n.getClassReflector();
    }
    else if (status.peek()==SUFFIX) {

    }
    else {
      throw new Error();
    }

  }  

}
