/*
 * File         : ConstructTranslator.java
 * Author       : Guansong Zhang
 * Created      : Tue Oct 27 11:53:19 1998
 * Revision     : $Revision: 1.15 $
 * Updated      : $Date: 1999/03/17 18:08:18 $
 */

package hpjava.compiler.translator;

import hpjava.compiler.util.*;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

import hpjava.compiler.unparser.*;

import hpjava.compiler.reflection.*;
import hpjava.compiler.abstractexp.*;
import hpjava.compiler.wrapper.*;

public class ConstructTranslator {

  Node target;

  public ConstructTranslator(OnStatement n) {
    target = n;    
  }

  public ConstructTranslator(OverallStatement n) {
    target = n;    
  }

  public ConstructTranslator(AtStatement n) {
    target = n;    
  }

  public void process() {
    ConstructProcessor processor = new ConstructProcessor();
    target.accept(processor);
  }

}

class ConstructProcessor extends DepthFirstVisitor {
  final int DEFAULT =0;
  final int FORON = 1;

  IntStack status = new IntStack();

  ConstructProcessor() {
  }

    // constructor has side effects on its input. 
    // a copy should be used?
    // if not, the node should be deleted.

  private BlockStatement groupDec(Expression group) {
    TypeDescriptor type = 
      new TypeDescriptor(new NodeToken("Group"));

    VariableDeclaratorDescriptor var = 
      new VariableDeclaratorDescriptor(new VariableDeclaratorId(Mixer.group(),
								new NodeListOptional()));
    var.setInitializer(group);

    LocalVariableDeclarationDescriptor dec = 
      new LocalVariableDeclarationDescriptor(type.leaf, var.leaf);


    NodeSequence sequence = new NodeSequence(2);
    sequence.addNode(dec.leaf);
    sequence.addNode(new NodeToken(";"));

    return  new BlockStatement(new NodeChoice(sequence, 0));;
  }

  private BlockStatement locationDecl(NodeToken id, Expression location) {
    TypeDescriptor type = new TypeDescriptor(new NodeToken("Location"));

    /* VariableDeclaratorDescriptor var = 
      new VariableDeclaratorDescriptor(new VariableDeclaratorId(Mixer.group(),
								new NodeListOptional())); */

    VariableDeclaratorDescriptor var = 
      new VariableDeclaratorDescriptor(new VariableDeclaratorId(id,
								new NodeListOptional()));
    var.setInitializer(location);

    LocalVariableDeclarationDescriptor dec = 
      new LocalVariableDeclarationDescriptor(type.leaf, var.leaf);

    NodeSequence sequence = new NodeSequence(2);
    sequence.addNode(dec.leaf);
    sequence.addNode(new NodeToken(";"));

    return  new BlockStatement(new NodeChoice(sequence, 0));;
  }

  public void visit (AtStatement n) {
	System.out.println(n.getParent(4).toString());
    NodeListOptional optional = new NodeListOptional();

    NamedLocationListDescriptor descriptor =
      new NamedLocationListDescriptor (n.f2);

	// To generate a list of location declaration statement. i.e.:
	// we have at(i = ... ,j = ... ,k = ...) in HPJava language
	// after translation, it becomes:
	// Location i = ...; // Generated by this method;
	// Location j = ...; // Generated by this method;
	// Location k = ...; // Generated by this method;
    // if( i.at() ) {
    //  if( j.at() ) {
    //   if( k.at() ) {
    //    ... 
    //   } k.ta();
    //  } j.ta(); 
    // } i.ta();
	// Rules used here:
	// FieldDeclaration ::= Type VariableDeclarator
	// Variable Declarator ::= <IDENTIFIER> = VariableInitializer;
    for (int i = 0; i<descriptor.numberOfLocs(); i++) {
      NodeToken id = descriptor.name(i);
      PrimaryExpression locationExpr = descriptor.loc(i).f2;

      AbstractPrimaryExpression apExpr = PrimaryExpressionManager.lookup(locationExpr);

      optional.addNode(locationDecl(id, apExpr.toExpression()));
    }

	// To generate nested "at" construct i.e. Before the translation, 
    // we have at(i = ... ,j = ... ,k = ...) {...} in HPJava language
    // after translation, it becomes:
	// Location i = ...;
    // Location j = ...;
    // Location k = ...;
	// if( i.at() ) {   // Generated by this method
	//	if( j.at() ) {  // Generated by this method
	//	 if( k.at() ) { // Generated by this method
	//	  ... 			// Original statements in "at" construct are kept.
    //   } k.ta(); 		// Generated by this method
    //  } j.ta();  		// Generated by this method
    // } i.ta();   		// Generated by this method
	// The nested construct is generated from the inner most level to outmost level

	Statement stat;
	BlockStatement blockStat; 
	NodeSequence sequence = new NodeSequence(1); 

	Statement nestedStmt = n.f4;
    for (int i = descriptor.numberOfLocs()-1; i >= 0; i--) {

      NodeToken id = descriptor.name(i);
	  AbstractPrimaryExpression aexp = new IdentifierDescriptor(id);
	  aexp = aexp.append(new IdentifierDescriptor(new NodeToken("at")));
	  aexp = aexp.append(new ArgumentsDescriptor());

      IfStatement ifStat = new IfStatement(new NodeToken("if"),
              n.f1,
              aexp.toAbstractExpression().toExpression(),
              n.f3,
              nestedStmt,
              new NodeOptional()); //NodeOptional is "else" here

	  stat = new Statement(new NodeChoice(ifStat, 5));
      blockStat = new BlockStatement(new NodeChoice(stat,1));
	  nestedStmt = new Statement(new NodeChoice(blockStat, 1));

      optional.addNode(nestedStmt);

      aexp = new IdentifierDescriptor(id);
	  aexp = aexp.append(new IdentifierDescriptor(new NodeToken("ta")));
	  aexp = aexp.append(new ArgumentsDescriptor());

	  sequence = new NodeSequence(2);
      sequence.addNode(aexp.toAbstractExpression().toStatementExpression());
      sequence.addNode(new NodeToken(";"));

      stat = new Statement(new NodeChoice(sequence,3));
      blockStat = new BlockStatement(new NodeChoice(stat,1));

      optional.addNode(blockStat);

    }

    Block block = new Block ( new NodeToken("{"),
                              optional,
                              new NodeToken("}") );

    block.show();

    n.getParent(4).insertAfter(block);
    n.getParent(4).destroy();
  }

  public void visit (OnStatement n) {
    NodeListOptional optional = new NodeListOptional();

    optional.addNode(groupDec(n.f2));

    AbstractPrimaryExpression aexp = new IdentifierDescriptor(Mixer.group());
    aexp = aexp.append(new IdentifierDescriptor(new NodeToken("on")));
    aexp = aexp.append(new ArgumentsDescriptor());

    IfStatement ifStat = 
      new IfStatement(new NodeToken("if"),
		      n.f1, 
		      aexp.toAbstractExpression().toExpression(),
		      n.f3, 
		      n.f4,
		      new NodeOptional());

    Statement stat = new Statement(new NodeChoice(ifStat, 5));
    BlockStatement blockStat = new BlockStatement(new NodeChoice(stat,1));
    
    optional.addNode(blockStat);

    aexp = new IdentifierDescriptor(Mixer.group());
    aexp = aexp.append(new IdentifierDescriptor(new NodeToken("no")));
    aexp = aexp.append(new ArgumentsDescriptor());

    NodeSequence sequence = new NodeSequence(2);
    sequence.addNode(aexp.toAbstractExpression().toStatementExpression());
    sequence.addNode(new NodeToken(";"));

    stat = new Statement(new NodeChoice(sequence,3));
    blockStat = new BlockStatement(new NodeChoice(stat,1));

    optional.addNode(blockStat);

    Block block = new Block ( new NodeToken("{"),
                              optional,
                              new NodeToken("}") );
    //block.show();

    //n.getParent(4).insertAfter(block);
    //n.getParent(4).destroy();
    stat = new Statement(new NodeChoice(block, 1));
    n.getParent(2).replaceWith(stat);
  }

  private BlockStatement rangeDec(NodeToken id, Expression range) {
    TypeDescriptor type = 
      new TypeDescriptor(new NodeToken("Range"));

    VariableDeclaratorDescriptor var = 
      new VariableDeclaratorDescriptor(new VariableDeclaratorId(Mixer.range(id),
								new NodeListOptional()));

    var.setInitializer(range);

    LocalVariableDeclarationDescriptor dec = 
      new LocalVariableDeclarationDescriptor(type.leaf, var.leaf);


    NodeSequence sequence = new NodeSequence(2);
    sequence.addNode(dec.leaf);
    sequence.addNode(new NodeToken(";"));

    return  new BlockStatement(new NodeChoice(sequence, 0));
  }

  private NamedLocation locAt(NodeToken id) {
    AbstractPrimaryExpression rng =
      new IdentifierDescriptor(Mixer.range(id));

    IdentifierDescriptor loc = 
      new IdentifierDescriptor (new NodeToken("location"));

    rng = rng.append(loc);

    IdentifierDescriptor idx =
      new IdentifierDescriptor(Mixer.index(id));

    ArgumentsDescriptor arg = 
      new ArgumentsDescriptor(idx);
    
    /*
    TripletOrExpressionListDescriptor idxExp =
      new TripletOrExpressionListDescriptor(new TripletOrExpressionList
                                            (new TripletOrExpression
                                             (new NodeChoice(idx.toExpression(), 1)),
                                             new NodeListOptional()));
    */

    rng = rng.append(arg);

    return new NamedLocation(id, new NodeToken("="), rng.toPrimaryExpression(), new NodeOptional());
  }

  private NodeOptional forInit(NodeToken id) {
    TypeDescriptor type = 
      new TypeDescriptor("int");

    VariableDeclaratorDescriptor var = 
      new VariableDeclaratorDescriptor(new VariableDeclaratorId(Mixer.index(id),
								new NodeListOptional()));

    Literal literal = new Literal( new NodeChoice( new NodeToken("0"), 0));

    LiteralDescriptor aexp = new LiteralDescriptor(literal);

    var.setInitializer(aexp.toExpression());

    LocalVariableDeclarationDescriptor dec = 
      new LocalVariableDeclarationDescriptor(type.leaf, var.leaf);

    NodeOptional optional = new NodeOptional();

    optional.addNode( new ForInit(new NodeChoice(dec.leaf, 0)) );

    return optional;
  }

 private NodeOptional forInit(NodeToken id , Expression initValExp) {
    TypeDescriptor type = 
      new TypeDescriptor("int");

    VariableDeclaratorDescriptor var = 
      new VariableDeclaratorDescriptor(new VariableDeclaratorId(Mixer.index(id),
								new NodeListOptional()));

    var.setInitializer(initValExp);

    LocalVariableDeclarationDescriptor dec = 
      new LocalVariableDeclarationDescriptor(type.leaf, var.leaf);

    NodeOptional optional = new NodeOptional();

    optional.addNode( new ForInit(new NodeChoice(dec.leaf, 0)) );

    return optional;
  }

  private NodeOptional forExpression(NodeToken id) {
    AbstractPrimaryExpression idx = 
      new IdentifierDescriptor(Mixer.index(id));

    AbstractPrimaryExpression rng = 
      new IdentifierDescriptor(Mixer.range(id));

    IdentifierDescriptor size = new IdentifierDescriptor(new NodeToken("size"));
    rng = rng.append(size);

    ArgumentsDescriptor arg = new ArgumentsDescriptor();
    rng = rng.append(arg);

    AbstractExpression condition = idx.toAbstractExpression();

    condition = condition.less(rng.toAbstractExpression());    

    NodeOptional optional = new NodeOptional();
    optional.addNode(condition.toExpression());

    return optional;
  }

 private NodeOptional forExpression(NodeToken id, Expression upperBound) {
    AbstractPrimaryExpression idx = 
      new IdentifierDescriptor(Mixer.index(id));

    /* AbstractExpression condition = idx.toAbstractExpression();

    condition = condition.less(upperBound.toAbstractExpression());

    NodeOptional optional = new NodeOptional();
	optional.addNode(condition.toExpression()); */

	/* ShiftExpression shftExpr0 = idx.toAbstractExpression().toExpression();
	NodeListOptional lessThan = new NodeListOptional(new NodeToken("<="));
	ShiftExpression shftExpr1 = upperBound.f0;
	lessThan.addNode(shftExpr1);
	RelationalExpression relatExp = new RelationalExpression(shftExpr0, lessThan);
	InstanceOfExpression instOfExpr = new InstanceOfExpression(relatExp,null);
	EqualityExpression eqExp = new EqualityExpression(instOfExpr, null);
	AndExpression andExpr = new AndExpression(eqExp, null);
	ExclusiveOrExpression exclOrExp = new ExclusiveOrExpression(andExpr, null);
	InclusiveOrExpression inclOrExp = new InclusiveOrExpression(exclOrExp, null);
	ConditionalAndExpression condAndExp = new ConditionalAndExpression(inclOrExp,null); 
	ConditionalOrExpression condOrExp = new ConditionalOrExpression(condAndExp,null);
	ConditionalExpression condExpression = new ConditionalExpression(condOrExp,null);
	Expression	expr = new Expression(condExpression, null); */

	AbstractExpression condition = idx.toAbstractExpression();

    condition = condition.less(ExpressionManager.lookup(upperBound));

    NodeOptional optional = new NodeOptional();
    optional.addNode(condition.toExpression());

    return optional;
  }

  private NodeOptional forUpdate(NodeToken id) {
    AbstractPrimaryExpression idx = 
      new IdentifierDescriptor(Mixer.index(id));

    AbstractExpression aexp = idx.toAbstractExpression();
    
    aexp = aexp.postfixAddition();

    StatementExpressionList list = 
      new StatementExpressionList(aexp.toStatementExpression(),
                                  new NodeListOptional());   

    ForUpdate update = new ForUpdate(list);


    NodeOptional optional = new NodeOptional();
    optional.addNode(update);

    return optional;

  }

 private NodeOptional forUpdate(NodeToken id, Expression updtExp) {
    AbstractPrimaryExpression idx = 
      new IdentifierDescriptor(Mixer.index(id));

    AbstractExpression aexp = idx.toAbstractExpression();

	AbstractExpression aexp1 = ExpressionManager.lookup(updtExp);

    aexp = aexp.additionAssignment(aexp1);

    StatementExpressionList list = 
      new StatementExpressionList(aexp.toStatementExpression(),
                                  new NodeListOptional());

    ForUpdate update = new ForUpdate(list);

    NodeOptional optional = new NodeOptional();
    optional.addNode(update);

    return optional;

  }

  // This method is modified by Qiang Zheng in order to fit the new grammar
  // of Overall statement.
  // Syntax of Overall Statement:
  // overall:  "overall" "(" NamedLocationList() ")" Statement()
  // NamedLocationList: NamedLocation() ( "," NamedLocation() )*
  // NamedLocation : <IDENTIFIER> "=" PrimaryExpression() [ "|" Triplet() ]

  public void visit (OverallStatement n) {
    NodeListOptional optional = new NodeListOptional();
    NodeListOptional lstOptional = new NodeListOptional();

    NamedLocationListDescriptor locList = 
      new NamedLocationListDescriptor(new NamedLocationList(null, new NodeListOptional())); 

    AtStatement atStatement = new AtStatement (new NodeToken("at"),
                                               new NodeToken("("),
                                               locList.leaf,
                                               new NodeToken(")"),
                                               n.f4);

    Statement statement = new Statement (new NodeChoice(atStatement, 16));
	BlockStatement blckStmt = new BlockStatement(new NodeChoice(statement,1));
	lstOptional.addNode(blckStmt);
	Block		blck = new Block(new NodeToken("{")
								, lstOptional
								, new NodeToken("}"));
	Statement	stmt = new Statement(new NodeChoice(blck,1));

    ForStatement forStatement = null;

    NamedLocationListDescriptor descriptor =
      new NamedLocationListDescriptor (n.f2);

    for (int i = 0; i<descriptor.numberOfLocs(); i++) {
      NodeToken id = descriptor.name(i);
      PrimaryExpression range = descriptor.base(i);

      NamedLocation ithLoc = descriptor.loc(i);

      AbstractPrimaryExpression apExp = PrimaryExpressionManager.lookup(range);

      optional.addNode(rangeDec(id, apExp.toExpression()));

      locList.addLoc(locAt(id));

	  // This is revised by Qiang Zheng
	  if (!ithLoc.f3.present()) {
        forStatement = new ForStatement (new NodeToken("for"),
                                         new NodeToken("("),
                                         forInit(id),
                                         new NodeToken(";"),
                                         forExpression(id),
                                         new NodeToken(";"),
                                         forUpdate(id),
                                         new NodeToken(")"),
                                         stmt);
      } else {
        NodeSequence namedLocOptSeq = (NodeSequence) ithLoc.f3.node;
		Triplet	namedLocTrip = (Triplet) namedLocOptSeq.elementAt(1);

		java.util.Vector childrenVec = namedLocTrip.getChildren();
		NodeOptional ndOpt = (NodeOptional) childrenVec.elementAt(0);
		Expression initExp = (Expression) ndOpt.node;
		ndOpt = (NodeOptional) childrenVec.elementAt(2);
		Expression upperValExp = (Expression) ndOpt.node;
		//Expression upperValExp = (Expression) childrenVec.elementAt(2).node;
		ndOpt = (NodeOptional) childrenVec.elementAt(3);
		NodeSequence updtSeqns = (NodeSequence) ndOpt.node;
		//NodeSequence updtSeqns = (NodeSequence) childrenVec.elementAt(3);
		Expression updateExp = (Expression) updtSeqns.elementAt(1);

        forStatement = new ForStatement (new NodeToken("for"),
                                         new NodeToken("("),
                                         forInit(id, initExp),
                                         new NodeToken(";"),
                                         forExpression(id,upperValExp),
                                         new NodeToken(";"),
                                         forUpdate(id,updateExp),
                                         new NodeToken(")"),
                                         stmt);
      }

      statement = new Statement (new NodeChoice(forStatement, 8));
    }

    BlockStatement blockStatement = 
      new BlockStatement( new NodeChoice(statement, 1));

    optional.addNode(blockStatement);

    Block block = new Block ( new NodeToken("{"),
                              optional,
                              new NodeToken("}") );

    //block.show();

    n.getParent(4).insertAfter(block);
    n.getParent(4).destroy();
	visit(atStatement);
  }

}

    /*
    System.err.println(n.parent.toString());
    System.err.println(n.getParent(1).toString());
    System.err.println(n.parent.parent.toString());
    System.err.println(n.getParent(2).toString());
    System.err.println(n.parent.parent.parent.toString());
    System.err.println(n.getParent(3).toString());
    System.err.println(n.parent.parent.parent.parent.toString());
    System.err.println(n.getParent(4).toString());
    System.err.println(n.parent.parent.parent.parent.parent.toString());
    System.err.println(n.getParent(5).toString());
    System.err.println(n.parent.parent.parent.parent.parent.parent.toString());
    System.err.println(n.getParent(6).toString());
    */
	      


/* In fact, we don't need visitor here. */
