/*
 * File         : Translator.java
 * Author       : Guansong Zhang
 * Created      : Thu May 14 12:03:02 1998
 * Revision     : $Revision: 1.21 $
 * Updated      : $Date: 1999/03/16 20:29:49 $
 */

package hpjava.compiler.translator;

import java.io.*;
import java.util.*;

import hpjava.compiler.util.*;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

import hpjava.compiler.reflection.*;
import hpjava.compiler.abstractexp.*;

import hpjava.compiler.unparser.*;

public class Translator {
  static public Node current;

  private Project prj;

  public Translator (Project prj) {
    this.prj=prj;
  }

  public void process() {
    Processor processor = new Processor();
    processor.visit(prj);
  }
}                                                                              

class Processor extends DepthFirstVisitor {

  public Processor() {
  }

  public void visit(Project prj) {
    
    Enumeration enum = prj.rootFileTable.keys();
    
  LOOP: while (enum.hasMoreElements()) {
      
      Node root = (Node) enum.nextElement();      

      root.accept(this);
      
    }

  }

  private ImportDeclaration importDec(String pkg) {
    StringTokenizer tokenizer = 
      new StringTokenizer(pkg, ".");

    NameDescriptor name = new NameDescriptor();
    
    while (tokenizer.hasMoreTokens()) {
      name.addID(new NodeToken(tokenizer.nextToken()));
    }

    NodeSequence sequence = new NodeSequence(2);
    sequence.addNode(new NodeToken("."));
    sequence.addNode(new NodeToken("*"));

    NodeOptional optional = new NodeOptional();
    optional.addNode(sequence);

    ImportDeclaration result =
      new ImportDeclaration(new NodeToken("import"),
                            name.leaf,
                            optional,
                            new NodeToken(";"));

    return result;
  }

  public void visit(CompilationUnit n) {
    n.f1.addNode(importDec("hpjava.lang"));
    n.f1.addNode(importDec("hpjava.spmd"));

    super.visit(n);
  } 

  public void visit(FieldDeclaration n) {
    DeclarationTranslator translator 
      = new DeclarationTranslator(n);

    translator.process();

    super.visit(n);
  }

  public void visit(MethodDeclaration n) {
    DeclarationTranslator translator 
      = new DeclarationTranslator(n);

    translator.process();

    super.visit(n);
  }

  public void visit(LocalVariableDeclaration n) {
    DeclarationTranslator translator 
      = new DeclarationTranslator(n);

    translator.process();

    super.visit(n);
  }


  public void visit(Expression n) {
    AbstractExpression exp 
      = ExpressionManager.lookup(n);

    if (exp == null)
      n.getParent(1).accept( new ITreePrinter());
    
    AbstractExpressionTranslator translator = 
      new AbstractExpressionTranslator(exp);
    
    translator.process();

  }


  public void visit(StatementExpression n) {
    AbstractExpression exp 
      = ExpressionManager.lookup(n);

    if (exp == null)
      n.getParent(1).accept( new ITreePrinter());

    AbstractExpressionTranslator translator = 
      new AbstractExpressionTranslator(exp);
    
    translator.process();

  }

  public void visit(GroupExpression n) {
    AbstractExpression exp 
      = ExpressionManager.lookup(n);

    if (exp == null)
      n.getParent(1).accept( new ITreePrinter());

    AbstractExpressionTranslator translator = 
      new AbstractExpressionTranslator(exp);
    
    translator.process();

  }

  public void visit (OnStatement n) { 
    ConstructTranslator translator = 
      new ConstructTranslator (n); 

    translator.process();

    super.visit(n);
  }

  public void visit (AtStatement n) { 
    ConstructTranslator translator = 
      new ConstructTranslator (n); 

    translator.process();

    super.visit(n);
  }

  public void visit (OverallStatement n) { 
    ConstructTranslator translator = 
      new ConstructTranslator (n); 

    translator.process();

    super.visit(n);
  }

  // Override the method visit()
   public void visit( NodeList n ) {
	int i = 0;
	while(  i < n.size() ) {
		((Node)n.elementAt(i)).accept(this);
		i++;
	}	
      // for ( Enumeration e = n.elements(); e.hasMoreElements(); )
      //   ((Node)e.nextElement()).accept(this);
   }

   public void visit(NodeListOptional n) {
	int i = 0;

      if ( n.present() )
		while(i < n.size() ) {
			((Node)n.elementAt(i)).accept(this);
			i++;
		}	
         // for ( Enumeration e = n.elements(); e.hasMoreElements(); )
         //   ((Node)e.nextElement()).accept(this);
   }
}
