//package hpjava.compiler.abstractexp;

package hpjava.compiler.unparser;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.abstractexp.*;

//import hpjava.compiler.unparser.*;

import java.util.*;
import java.io.*;

public class AbstractExpressionTreePrinter extends AbstractExpressionVisitor {
  protected PrintWriter out;
  protected Space spc = new Space(2); 

  protected ITreePrinter tp;

  public AbstractExpressionTreePrinter(ITreePrinter itp){
    this.tp = itp;
    this.out = itp.out;
    this.spc = itp.spc;
  }

  public AbstractExpressionTreePrinter() {
    out = new PrintWriter(System.out, true);
  }

  public AbstractExpressionTreePrinter(Writer o) {
    out = new PrintWriter(o, true);
  }

  public AbstractExpressionTreePrinter(OutputStream o) {
    out = new PrintWriter(o, true);
  }

  public void flushWriter() { 
    out.flush(); 
  }

  //
  // Auto class visitors--probably don't need to be overridden.
  //

  //
  // User-generated visitor methods below
  //

  //
  // f0 -> "="
  //       | "*="
  //       | "/="
  //       | "%="
  //       | "+="
  //       | "-="
  //       | "<<="
  //       | ">>="
  //       | ">>>="
  //       | "&="
  //       | "^="
  //       | "|="
  //
  public void visit(AssignmentNode n) {
    out.print(spc.spc);
    out.print("AssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(AdditionAssignmentNode n) {
    out.print(spc.spc);
    out.print("AdditiveAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(SubtractionAssignmentNode n) {
    out.print(spc.spc);
    out.print("SubtractionAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(MultiplicationAssignmentNode n) {
    out.print(spc.spc);
    out.print("MultiplicationAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(DivisionAssignmentNode n) {
    out.print(spc.spc);
    out.print("DivisionAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(AndAssignmentNode n) {
    out.print(spc.spc);
    out.print("AndAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(InclusiveOrAssignmentNode n) {
    out.print(spc.spc);
    out.print("InclusiveOrAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(ExclusiveOrAssignmentNode n) {
    out.print(spc.spc);
    out.print("ExclusiveOrAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(LeftShiftAssignmentNode n) {
    out.print(spc.spc);
    out.print("LeftShiftAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(RightShiftAssignmentNode n) {
    out.print(spc.spc);
    out.print("RightShiftAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(ExtensionRightShiftAssignmentNode n) {
    out.print(spc.spc);
    out.print("ExtensionRightShiftAssignmentNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> ConditionalOrExpression()
  // f1 -> [ "?" Expression() ":" ConditionalExpression() ]
  //
  public void visit(ConditionalNode n) {
    out.print(spc.spc);
    out.print("ConditionalNode");
    out.println();

    spc.updateSpc(+1);
    n.condition.accept(this);
    n.trueResult.accept(this);
    n.falseResult.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> ConditionalAndExpression()
  // f1 -> ( "||" ConditionalAndExpression() )*
  //
  public void visit(ConditionalOrNode n) {
    out.print(spc.spc);
    out.print("ConditionalOrNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> InclusiveOrExpression()
  // f1 -> ( "&&" InclusiveOrExpression() )*
  //
  public void visit(ConditionalAndNode n) {
    out.print(spc.spc);
    out.print("ConditionalAndNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> AndExpression()
  // f1 -> ( "^" AndExpression() )*
  //
  public void visit(ExclusiveOrNode n) {
    out.print(spc.spc);
    out.print("ExclusiveOrNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> ExclusiveOrExpression()
  // f1 -> ( "|" ExclusiveOrExpression() )*
  //
  public void visit(InclusiveOrNode n) {
    out.print(spc.spc);
    out.print("InclusiveOrNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> InstanceOfExpression()
  // f1 -> ( ( "==" | "!=" ) InstanceOfExpression() )*
  //
  public void visit(EqualityNode n) {
    out.print(spc.spc);
    out.print("EqualityNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(InEqualityNode n) {
    out.print(spc.spc);
    out.print("InEqualityNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> RelationalExpression()
  // f1 -> [ "instanceof" Type() ]
  //
  public void visit(InstanceOfNode n) {
    out.print(spc.spc);
    out.print("InEqualityNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    n.type.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> ShiftExpression()
  // f1 -> ( ( "<" | ">" | "<=" | ">=" ) ShiftExpression() )*
  //
  public void visit(GreaterNode n) {
    out.print(spc.spc);
    out.print("GreaterNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(GreaterThanEqNode n) {
    out.print(spc.spc);
    out.print("GreaterThanNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(LessNode n) {
    out.print(spc.spc);
    out.print("LessNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(LessThanEqNode n) {
    out.print(spc.spc);
    out.print("LessThanNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> ShiftExpression()
  // f1 -> ( ( "<" | ">" | "<=" | ">=" ) ShiftExpression() )*
  //
  public void visit(LeftShiftNode n) {
    out.print(spc.spc);
    out.print("LeftShiftNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(RightShiftNode n) {
    out.print(spc.spc);
    out.print("RightShiftNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> AdditiveExpression()
  // f1 -> ( ( "<<" | ">>" | ">>>" ) AdditiveExpression() )*
  //
  public void visit(ExtensionRightShiftNode n) {
    out.print(spc.spc);
    out.print("ExtensionRightShiftNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> MultiplicativeExpression()
  // f1 -> ( ( "+" | "-" ) MultiplicativeExpression() )*
  //
  public void visit(AdditiveNode n) {
    out.print(spc.spc);
    out.print("AdditiveNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(SubtractiveNode n) {
    out.print(spc.spc);
    out.print("SubtractiveNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> UnaryExpression()
  // f1 -> ( ( "*" | "/" | "%" ) UnaryExpression() )*
  //
  public void visit(MultiplicativeNode n) {
    out.print(spc.spc);
    out.print("MultiplicativeNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(DivideNode n) {
    out.print(spc.spc);
    out.print("DivideNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(ModuleNode n) {
    out.print(spc.spc);
    out.print("ModuleNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> ( "+" | "-" ) UnaryExpression()
  //       | PreIncrementExpression()
  //       | PreDecrementExpression()
  //       | UnaryExpressionNotPlusMinus()
  //
  public void visit(PlusNode n) {
    out.print(spc.spc);
    out.print("PlusNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(MinusNode n) {
    out.print(spc.spc);
    out.print("MinusNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> EqualityExpression()
  // f1 -> ( "&" EqualityExpression() )*
  //
  public void visit(AndNode n) {
    out.print(spc.spc);
    out.print("AndNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> "(" Type() ")" UnaryExpression()
  //       | "(" Type() ")" UnaryExpressionNotPlusMinus()
  //
  public void visit(CastNode n) {
    out.print(spc.spc);
    out.print("CastNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    n.type.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> ( "~" | "!" ) UnaryExpression()
  //       | CastExpression()
  //       | PostfixExpression()
  //
  public void visit(ComplementNode n) {
    out.print(spc.spc);
    out.print("ComplementNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(LogicalNotNode n) {
    out.print(spc.spc);
    out.print("LogicalNotNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> "++"
  // f1 -> PrimaryExpression()
  //
  public void visit(PreIncrementNode n) {
    out.print(spc.spc);
    out.print("PreIncrementNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> "--"
  // f1 -> PrimaryExpression()
  //
  public void visit(PreDecrementNode n) {
    out.print(spc.spc);
    out.print("PreDecrementNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> PrimaryExpression()
  // f1 -> [ "++" | "--" ]
  //
  public void visit(PostfixNode n) {
    out.print(spc.spc);
    out.print("PostfixNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(PostfixAdditionNode n) {
    out.print(spc.spc);
    out.print("PostfixAdditionNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(PostfixSubtractionNode n) {
    out.print(spc.spc);
    out.print("PostfixSubtractionNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this);
    spc.updateSpc(-1);
  }

  //
  // f0 -> PrimaryExpression()
  // f1 -> ( "/" PrimaryExpression() )*
  //
  public void visit(GroupNode n) {
    out.print(spc.spc);
    out.print("GroupNode");
    out.println();

    spc.updateSpc(+1);
    n.left.accept(this);
    n.right.accept(this);
    spc.updateSpc(-1);
  }

  public void visit(PrimaryExpressionNode n) {
    out.print(spc.spc);
    out.print("PrimaryExpressionNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this.tp);
    spc.updateSpc(-1);
  }

  public void visit(TypeNode n) {
    out.print(spc.spc);
    out.print("TypeNode");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(this.tp);
    spc.updateSpc(-1);
  }

}
