package hpjava.compiler.unparser;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.abstractexp.*;
import hpjava.compiler.util.*;

//import hpjava.compiler.unparser.*;

import java.util.*;
import java.io.*;

public class AbstractExpressionUnparsePrinter 
  extends AbstractExpressionVisitor {

  final int PREFIX = 0;
  final int SUFFIX = 1;

  protected PrintWriter out;

  protected int posCounter [];

//  public int lineCounter = 1;
//  public int columnCounter = 1;

  protected IUnparsePrinter iup;  

  private IntStack status=new IntStack(); {
    // for simple expression with out concatenation, it is a prefix.
    status.push(PREFIX);
  }

  public AbstractExpressionUnparsePrinter(IUnparsePrinter iup){
    this.iup = iup;
    this.out = iup.out;
    this.posCounter = iup.posCounter;
  }

  public AbstractExpressionUnparsePrinter() { 
    this.out = new PrintWriter(System.out, true); 

    this.posCounter = new int [2];

    this.posCounter[0]=1;
    this.posCounter[1]=1;    
  }

  public AbstractExpressionUnparsePrinter(Writer o) { 
    this.out = new PrintWriter(o, true); 

    this.posCounter = new int [2];

    this.posCounter[0]=1;
    this.posCounter[1]=1;    
  }

  public AbstractExpressionUnparsePrinter(OutputStream o) { 
    this.out = new PrintWriter(o, true); 

    this.posCounter = new int [2];

    this.posCounter[0]=1;
    this.posCounter[1]=1;    
  }

  public void flushWriter(){ out.flush(); }


  // write unparse here.

  public void visit(PrimaryExpressionNode n) {

    n.leaf.accept(iup);

  }
   
  public void visit(TypeNode n) {

    n.leaf.accept(iup);

  }


   public void visit(AssignmentNode n) {
      String image = "=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(AdditionAssignmentNode n) {
      String image = "+=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(SubtractionAssignmentNode n) {
      String image = "-=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(MultiplicationAssignmentNode n) {
      String image = "*=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(DivisionAssignmentNode n) {
      String image = "/=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(AndAssignmentNode n) {
      String image = "&=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(InclusiveOrAssignmentNode n) {
      String image = "|=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(ExclusiveOrAssignmentNode n) {
      String image = "^=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(LeftShiftAssignmentNode n) {
      String image = "<<=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(RightShiftAssignmentNode n) {
      String image = ">>=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(ModulusAssignmentNode n) {
      String image = "%=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   public void visit(ExtensionRightShiftAssignmentNode n) {
      String image = ">>>=";

      n.left.accept(this);
      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.right.accept(this);
   }

   //
   // f0 -> ConditionalOrExpression()
   // f1 -> [ "?" Expression() ":" ConditionalExpression() ]
   //
   public void visit(ConditionalNode n) {
      String image1 = "?";
      String image2 = ":";

      n.condition.accept(this);

      if(n.trueResult != null || n.falseResult != null){
        out.print(" " + image1);
        posCounter[1] += image1.length()+1; 
        n.trueResult.accept(this);

        out.print(" " + image2);
        posCounter[1] += image2.length()+1; 
        n.falseResult.accept(this);
      }
   }

   //
   // f0 -> ConditionalAndExpression()
   // f1 -> ( "||" ConditionalAndExpression() )*
   //
   public void visit(ConditionalOrNode n) {
      String image = "||";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> InclusiveOrExpression()
   // f1 -> ( "&&" InclusiveOrExpression() )*
   //
   public void visit(ConditionalAndNode n) {
      String image = "&&";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> AndExpression()
   // f1 -> ( "^" AndExpression() )*
   //
   public void visit(ExclusiveOrNode n) {
      String image = "^";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> ExclusiveOrExpression()
   // f1 -> ( "|" ExclusiveOrExpression() )*
   //
   public void visit(InclusiveOrNode n) {
      String image = "|";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> InstanceOfExpression()
   // f1 -> ( ( "==" | "!=" ) InstanceOfExpression() )*
   //
   public void visit(EqualityNode n) {
      String image = "==";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(InEqualityNode n) {
      String image = "!=";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> RelationalExpression()
   // f1 -> [ "instanceof" Type() ]
   //
   public void visit(InstanceOfNode n) {
      String image = "instanceof";

      n.leaf.accept(this);
 
      if(n.type != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.type.accept(this);
      }
   }

   //
   // f0 -> ShiftExpression()
   // f1 -> ( ( "<" | ">" | "<=" | ">=" ) ShiftExpression() )*
   //
   public void visit(GreaterNode n) {
      String image = ">";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(GreaterThanEqNode n) {
      String image = ">=";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(LessNode n) {
      String image = "<";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(LessThanEqNode n) {
      String image = "<=";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> AdditiveExpression()
   // f1 -> ( ( "<<" | ">>" | ">>>" ) AdditiveExpression() )*
   //
   public void visit(ExtensionRightShiftNode n) {
      String image = ">>>";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(LeftShiftNode n) {
      String image = "<<";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(RightShiftNode n) {
      String image = ">>";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> MultiplicativeExpression()
   // f1 -> ( ( "+" | "-" ) MultiplicativeExpression() )*
   //
   public void visit(AdditiveNode n) {
      String image = "+";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(SubtractiveNode n) {
      String image = "-";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> UnaryExpression()
   // f1 -> ( ( "*" | "/" | "%" ) UnaryExpression() )*
   //
   public void visit(MultiplicativeNode n) {
      String image = "*";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(DivideNode n) {
      String image = "/";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   public void visit(ModuleNode n) {
      String image = "%";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> ( "+" | "-" ) UnaryExpression()
   //       | PreIncrementExpression()
   //       | PreDecrementExpression()
   //       | UnaryExpressionNotPlusMinus()
   //
   public void visit(PlusNode n) {
      String image = "+";

      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.leaf.accept(this);
   }

   public void visit(MinusNode n) {
      String image = "-";

      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.leaf.accept(this);
   }

   //
   // f0 -> EqualityExpression()
   // f1 -> ( "&" EqualityExpression() )*
   //
   public void visit(AndNode n) {
      String image = "&";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

   //
   // f0 -> "(" Type() ")" UnaryExpression()
   //       | "(" Type() ")" UnaryExpressionNotPlusMinus()
   //
   public void visit(CastNode n) {
      String image1 = "(";
      String image2 = ")";

      out.print(" " + image1);
      posCounter[1] += image1.length()+1; 
      n.type.accept(this);

      out.print(image2);
      posCounter[1] += image2.length()+1; 
      n.leaf.accept(this);
   }

   //
   // f0 -> ( "~" | "!" ) UnaryExpression()
   //       | CastExpression()
   //       | PostfixExpression()
   //
   public void visit(ComplementNode n) {
      String image = "~";

      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.leaf.accept(this);
   }

   public void visit(LogicalNotNode n) {
      String image = "!";

      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.leaf.accept(this);
   }

   //
   // f0 -> "++"
   // f1 -> PrimaryExpression()
   //
   public void visit(PreIncrementNode n) {
      String image = "++";

      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.leaf.accept(this);
   }

   //
   // f0 -> "--"
   // f1 -> PrimaryExpression()
   //
   public void visit(PreDecrementNode n) {
      String image = "--";

      out.print(" " + image);
      posCounter[1] += image.length()+1; 
      n.leaf.accept(this);
   }

   //
   // f0 -> PrimaryExpression()
   // f1 -> [ "++" | "--" ]
   //
   public void visit(PostfixNode n) {
      String image = "++";

      n.leaf.accept(this);
      out.print(image);
      posCounter[1] += image.length(); 
   }

   public void visit(PostfixAdditionNode n) {
      String image = "++";

      n.leaf.accept(this);
      out.print(image);
      posCounter[1] += image.length(); 
   }

   public void visit(PostfixSubtractionNode n) {
      String image = "--";

      n.leaf.accept(this);
      out.print(image);
      posCounter[1] += image.length(); 
   }

   //
   // f0 -> PrimaryExpression()
   // f1 -> ( "/" PrimaryExpression() )*
   //
   public void visit(GroupNode n) {
      String image = "/";

      n.left.accept(this);
 
      if(n.right != null){
        out.print(" " + image);
        posCounter[1] += image.length()+1; 

        n.right.accept(this);
      }
   }

}
