package hpjava.compiler.unparser;

import hpjava.compiler.syntaxtree.*;

import hpjava.compiler.abstractexp.*;
import hpjava.compiler.util.*;

import java.util.*;
import java.io.*;

public class AbstractPrimaryExpressionPrettyPrinter 
  extends AbstractPrimaryExpressionVisitor 
{
  final int PREFIX = 0;
  final int SUFFIX = 1;

  protected PrintWriter out;
  protected Space spc ;

  private IntStack status=new IntStack(); {
    // for simple expression with out concatenation, it is a prefix.
    status.push(PREFIX);
  }
  
  public AbstractPrimaryExpressionPrettyPrinter(IPrettyPrinter ipp){
    this.out = ipp.out;
    this.spc = ipp.spc;
  }

  public AbstractPrimaryExpressionPrettyPrinter() { 
    this.out = new PrintWriter(System.out, true); 
    spc = new Space(3);
  }
  
  public AbstractPrimaryExpressionPrettyPrinter(Writer o) { 
    this.out = new PrintWriter(o, true); 
    spc = new Space(3);
  }
  
  public AbstractPrimaryExpressionPrettyPrinter(OutputStream o) { 
    this.out = new PrintWriter(o, true); 
    spc = new Space(3);
  }
  
  public void flushWriter() { 
    out.flush(); 
  }

  public void visit (PrimaryConcatenation n) {
    status.push(PREFIX);
    n.prefix.accept(this);
    status.pop();

    status.push(SUFFIX);
    n.suffix.accept(this);
    status.pop();
  }
  

  public void visit (ThisDescriptor n) {
    String image = "this";

    if (status.peek()==PREFIX) {
      out.print(image);
    }
    else {
      out.print("."+image);
    }
  }

  public void visit (SuperDescriptor n) {
    String image = "super";

    if (status.peek()==PREFIX) {
      out.print(image);
    }
    else {
      throw new Error();
    }
  }
  
  public void visit (LiteralDescriptor n) {
    if (status.peek()==PREFIX) {
      n.leaf.accept(new IPrettyPrinter(this));
    }
    else {
      throw new Error();
    }
  }

  public void visit (ClassLiteralDescriptor n) {
    String image = ".class";

    if (status.peek()==PREFIX) {
      n.leaf.accept(new IPrettyPrinter(this));
      out.print(image);
    }
    else {
      throw new Error();
    }
  }

  public void visit (TripletOrExpressionListDescriptor n) {
    if (status.peek()==PREFIX) {
      throw new Error();
    }
    else {
      out.print("[");
      n.leaf.accept(new IPrettyPrinter(this));
      out.print("]");
    }
  }

  public void visit (DistArraySecRefDimListDescriptor n) {
    if (status.peek()==PREFIX) {
      throw new Error();
    }
    else {
      out.print("[[");
      n.leaf.accept(new IPrettyPrinter(this));
      out.print("]]");
    }
  }

  public void visit (IdentifierDescriptor n) {
    if (status.peek()==PREFIX) {
      n.leaf.accept(new IPrettyPrinter(this));
    }
    else {
      out.print(".");
      n.leaf.accept(new IPrettyPrinter(this));
    }
  }

  public void visit (AllocationExpressionDescriptor n) {
    if (status.peek()==PREFIX) {
      n.leaf.accept(new IPrettyPrinter(this));
    }
    else {
      out.print(".");
      n.leaf.accept(new IPrettyPrinter(this));
    }
  }

  public void visit (ExpressionDescriptor n) {
    if (status.peek()==PREFIX) {
      out.print("(");
      n.leaf.accept(new IPrettyPrinter(this));
      out.print(")");
    }
    else {
      throw new Error();
    }
  }

  public void visit (ArgumentsDescriptor n) {
    if (status.peek()==PREFIX) {
      throw new Error();
    }
    else {
      n.leaf.accept(new IPrettyPrinter(this));
    }
  }

}
