/*
 * File         : AbstractPrimaryExpressionTreePrinter.java
 * Author       : Guansong Zhang
 * Created      : Fri Oct  9 14:53:48 1998
 * Revision     : $Revision: 1.9 $
 * Updated      : $Date: 1999/03/16 20:29:54 $
 */

package hpjava.compiler.unparser;

import hpjava.compiler.syntaxtree.*;

import hpjava.compiler.abstractexp.*;

import java.util.*;
import java.io.*;

public class AbstractPrimaryExpressionTreePrinter
  extends AbstractPrimaryExpressionVisitor
{
  protected PrintWriter out;
  protected Space spc ;

  public AbstractPrimaryExpressionTreePrinter(ITreePrinter itp){
    this.out = itp.out;
    this.spc = itp.spc;
  }

  public AbstractPrimaryExpressionTreePrinter() {
    out = new PrintWriter(System.out, true);
    spc = new Space(2);
  }

  public AbstractPrimaryExpressionTreePrinter(Writer o) {
    out = new PrintWriter(o, true);
    spc = new Space(2);
  }

  public AbstractPrimaryExpressionTreePrinter(OutputStream o) {
    out = new PrintWriter(o, true);
    spc = new Space(2);
  }

  public void flushWriter() { 
    out.flush(); 
  }

  public void visit (PrimaryConcatenation n) {
    out.print(spc.spc);
    out.print("PrimaryConcatenation");
    out.println();

    spc.updateSpc(+1);
    n.prefix.accept(this);
    n.suffix.accept(this);
    spc.updateSpc(-1);
  }

  public void visit (ThisDescriptor n) {
    out.print(spc.spc);
    out.print("ThisDescriptor");
    out.println();
  }

  public void visit (SuperDescriptor n) {
    out.print(spc.spc);
    out.print("SuperDescriptor");
    out.println();
  }

  public void visit (LiteralDescriptor n) {
    out.print(spc.spc);
    out.print("LiteralDescriptor");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(new ITreePrinter(this));
    spc.updateSpc(-1);
  }

  public void visit (ClassLiteralDescriptor n) {
    out.print(spc.spc);
    out.print("ClassLiteralDescriptor");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(new ITreePrinter(this));
    spc.updateSpc(-1);
  }

  public void visit (TripletOrExpressionListDescriptor n) {
    out.print(spc.spc);
    out.print("TripletOrExpressionListDescriptor");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(new ITreePrinter(this));
    spc.updateSpc(-1);
  }

  public void visit (DistArraySecRefDimListDescriptor n) {
    out.print(spc.spc);
    out.print("DistArraySecRefDimListDescriptor");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(new ITreePrinter(this));
    spc.updateSpc(-1);
  }

  public void visit (IdentifierDescriptor n) {
    out.print(spc.spc);
    out.print("IdentifierDescriptor");
    out.println();

    spc.updateSpc(+1);
    out.print(spc.spc);
    n.leaf.accept(new ITreePrinter(this));
    out.println();
    spc.updateSpc(-1);
  }

  public void visit (AllocationExpressionDescriptor n) {
    out.print(spc.spc);
    out.print("AllocationExpressionDescriptor");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(new ITreePrinter(this));
    spc.updateSpc(-1);
  }

  public void visit (ExpressionDescriptor n) {
    out.print(spc.spc);
    out.print("ExpressionDescriptor");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(new ITreePrinter(this));
    spc.updateSpc(-1);
  }

  public void visit (ArgumentsDescriptor n) {
    out.print(spc.spc);
    out.print("ArgumentsDescriptor");
    out.println();

    spc.updateSpc(+1);
    n.leaf.accept(new ITreePrinter(this));
    spc.updateSpc(-1);
  }

}
