/*
 * File         : AbstractPrimaryExpressionUnparsePrinter.java
 * Author       : Guansong Zhang
 * Created      : Fri Oct  9 14:53:48 1998
 * Revision     : $Revision: 1.6 $
 * Updated      : $Date: 1999/03/16 20:29:55 $
 */

package hpjava.compiler.unparser;

import hpjava.compiler.syntaxtree.*;

import hpjava.compiler.abstractexp.*;
import hpjava.compiler.util.*;

import java.util.*;
import java.io.*;

public class AbstractPrimaryExpressionUnparsePrinter 
  extends AbstractPrimaryExpressionVisitor 
{
  final int PREFIX = 0;
  final int SUFFIX = 1;

  protected PrintWriter out;

  protected int posCounter [];

  //protected int lineCounter;
  //protected int columnCounter;

  protected NodeToken previous = null;
  
  private IntStack status=new IntStack(); {
    // for simple expression with out concatenation, it is a prefix.
    status.push(PREFIX);
  }
  
  public AbstractPrimaryExpressionUnparsePrinter(IUnparsePrinter iup){
    this.out = iup.out;

    this.previous = iup.previous;

    this.posCounter = iup.posCounter;
  }

  public AbstractPrimaryExpressionUnparsePrinter() { 
    this.out = new PrintWriter(System.out, true); 

    this.posCounter = new int [2];

    this.posCounter[0]=1;
    this.posCounter[1]=1;    
  }
  
  public AbstractPrimaryExpressionUnparsePrinter(Writer o) { 
    this.out = new PrintWriter(o, true); 

    this.posCounter = new int [2];

    this.posCounter[0]=1;
    this.posCounter[1]=1;    

  }
  
  public AbstractPrimaryExpressionUnparsePrinter(OutputStream o) { 
    this.out = new PrintWriter(o, true); 

    this.posCounter = new int [2];

    this.posCounter[0]=1;
    this.posCounter[1]=1;    
  }
  
  public void flushWriter() { 
    out.flush(); 
  }

  public void visit (PrimaryConcatenation n) {
    status.push(PREFIX);
    n.prefix.accept(this);
    status.pop();

    status.push(SUFFIX);
    n.suffix.accept(this);
    status.pop();
  }
  

  public void visit (ThisDescriptor n) {
    String image = "this";

    if (status.peek()==PREFIX) {
      out.print(image);
      posCounter[1] += image.length();
    }
    else {
      out.print("."+image);
      posCounter[1] += image.length()+1;
    }
  }

  public void visit (SuperDescriptor n) {
    String image = "super";

    if (status.peek()==PREFIX) {
      out.print(image);
      posCounter[1] += image.length();
    }
    else {
      throw new Error();
    }
  }
  
  public void visit (LiteralDescriptor n) {
    if (status.peek()==PREFIX) {
      n.leaf.accept(new IUnparsePrinter(this));
    }
    else {
      throw new Error();
    }
  }

  public void visit (ClassLiteralDescriptor n) {
    String image = ".class";

    if (status.peek()==PREFIX) {
      n.leaf.accept(new IUnparsePrinter(this));
      out.print(image);
      posCounter[1] += image.length();
    }
    else {
      throw new Error();
    }
  }

  public void visit (TripletOrExpressionListDescriptor n) {
    if (status.peek()==PREFIX) {
      throw new Error();
    }
    else {
      out.print("[");

      previous = new NodeToken("[");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.LBRACKET;
      posCounter[1]++;

      n.leaf.accept(new IUnparsePrinter(this));

      out.print("]");

      previous = new NodeToken("]");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.RBRACKET;
      posCounter[1]++;

    }
  }

  public void visit (DistArraySecRefDimListDescriptor n) {
    if (status.peek()==PREFIX) {
      throw new Error();
    }
    else {
      out.print("[[");

      previous = new NodeToken("[");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.LBRACKET;
      posCounter[1]++;      
      posCounter[1]++;

      n.leaf.accept(new IUnparsePrinter(this));

      out.print("]]");

      previous = new NodeToken("]");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.RBRACKET;
      posCounter[1]++;
      posCounter[1]++;
    }
  }

  public void visit (IdentifierDescriptor n) {
    if (status.peek()==PREFIX) {
      n.leaf.accept(new IUnparsePrinter(this));
    }
    else {
      out.print(".");

      previous = new NodeToken(".");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.DOT;
      posCounter[1] ++;

      n.leaf.accept(new IUnparsePrinter(this));
    }
  }

  public void visit (AllocationExpressionDescriptor n) {
    if (status.peek()==PREFIX) {
      n.leaf.accept(new IUnparsePrinter(this));
    }
    else {
      out.print(".");

      previous = new NodeToken(".");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.DOT;
      posCounter[1] ++;

      n.leaf.accept(new IUnparsePrinter(this));
    }
  }

  public void visit (ExpressionDescriptor n) {
    if (status.peek()==PREFIX) {
      out.print("(");

      previous = new NodeToken("(");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.LPAREN;
      posCounter[1] ++;

      n.leaf.accept(new IUnparsePrinter(this));

      out.print(")");

      previous = new NodeToken(")");
      previous.kind = hpjava.compiler.parser.HPJavaParserConstants.LPAREN;
      posCounter[1] ++;
    }
    else {
      throw new Error();
    }
  }

  public void visit (ArgumentsDescriptor n) {
    if (status.peek()==PREFIX) {
      throw new Error();
    }
    else {
      n.leaf.accept(new IUnparsePrinter(this));
      //out.print("|");
    }
  }

}
