package hpjava.compiler.unparser;

import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;
import hpjava.compiler.parser.*;

import java.util.*;
import java.io.*;

public class UnparsePrinter extends DepthFirstVisitor {
  protected PrintWriter out;

  //protected int lineCounter;
  //protected int columnCounter;

  // use array as position counter;
  protected int posCounter [];

  // used to remember previous token.
  protected NodeToken previous = null;

  public UnparsePrinter() { 
    out = new PrintWriter(System.out, true); 

    posCounter = new int [2];

    posCounter[0]=1;
    posCounter[1]=1;    
  }
  
    
  public UnparsePrinter(Writer o) { 
    out = new PrintWriter(o, true); 

    posCounter = new int [2];

    posCounter[0]=1;
    posCounter[1]=1;    
  }

  public UnparsePrinter(OutputStream o)  {
    out = new PrintWriter(o, true); 

    posCounter = new int [2];

    posCounter[0]=1;
    posCounter[1]=1;    
  }

  public void flushWriter()     { out.flush(); }

  //public int posCounter[0] = 1;
  //public int posCounter[1] = 1;

  //
  // Auto class visitors--probably don't need to be overridden.
  //

  public void visit(NodeToken n) {
    int tmpColumn;

    while(n.beginLine > posCounter[0]){
      flushWriter();
      out.println();
      posCounter[0]++;
      posCounter[1] = 1;
    }      

    if (n.beginColumn < posCounter[1]) {
      // if this is a seperator or operator, no space is needed.
      // else put a space before the current token
      if (previous != null) {
        if (previous.kind >= HPJavaParserConstants.LPAREN
            && previous.kind <= HPJavaParserConstants.RUNSIGNEDSHIFTASSIGN) {
          out.print(n.tokenImage);
          posCounter[1] += n.tokenImage.length();
        } 
        else if (n.kind >= HPJavaParserConstants.LPAREN 
            && n.kind <= HPJavaParserConstants.RUNSIGNEDSHIFTASSIGN){
          out.print(n.tokenImage); 
          posCounter[1] += n.tokenImage.length(); 
        }
        else {
          out.print(" "); 
          out.print(n.tokenImage);  
          posCounter[1] += n.tokenImage.length()+1; 
        }
      }
      else {
        out.print(" ");
        out.print(n.tokenImage); 
        posCounter[1] += n.tokenImage.length()+1;
      }
    }
    else {
      while(n.beginColumn > posCounter[1]) {
        out.print(" ");
        posCounter[1]++;
      }

      posCounter[1] += n.tokenImage.length();
      out.print(n.tokenImage); 
    }

    flushWriter();
    
    this.previous = n;
    
  }

  //
  // User-generated visitor methods below
  //

  //
  // f0 -> [ PackageDeclaration() ]
  // f1 -> ( ImportDeclaration() )*
  // f2 -> ( TypeDeclaration() )*
  // f3 -> < EOF >
  //
  public void visit(CompilationUnit n) {
    n.f0.accept(this);
    n.f1.accept(this);
    n.f2.accept(this);

    out.println();
    flushWriter();
  }

}


